(*
 *   FeedReader
 *   Copyright (C) 2003 The FeedReader Developement Group
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *)
unit FReaderProperties;

interface

uses
  Forms;

Const
  DefaultFeedRefreshIdValue = 14;
  //Change this if needed for branding
  //BRANDING
  //Also include file import.xml (same format as subscriptions.xml is) to include custom feedlist
  ProgramName = 'Feedreader';
  MiniBrowserWelcomeText = '';
  DefaultArchiveSizeIdValue = 8888;
  UnlimitedArchiveSizeIdValue = 9999;

  PropertiesFileName = ProgramName+'.ini';

  Feedretrieval_Timeout = 1200000;  // Time in msec, that TIdHTTP waits for a server response 

Const
  gUserAgents: array[0..6] Of String
    = (ProgramName,
       'Mozilla/4.0 (compatible; MSIE 5.5; Windows 2000)',
       'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)',
       'Mozilla/4.0 (compatible; MSIE 5.5; Windows 98)',
       'Mozilla/4.79 [en] (Windows NT 5.0; U)',
       'Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.4) Gecko/20011128 Netscape6/6.2.1',
       'Opera/6.01 (Windows 2000; U)  [en]' );


Type
  TShowPopupsMode = (POP_Always, POP_Never, POP_Ask);

  TFeedReaderProperties = Class( TObject )
  Private
    FApplicationTitle: String;
    FApplicationDirectory: string;
    FUseIEProxySettings: Boolean;

    Function GetDataDirectory: String;
    Function GetRunAtStartup: Boolean;
    Procedure SetRunAtStartup( Value: Boolean );
    Procedure SetUseIEProxySettings( Value: Boolean );
    procedure SetApplicationDirectory(const Value: string);
    function GetAppVersion: string;
  Public
    Constructor Create; virtual;
    Destructor Destroy; override;

    Procedure Load;
    Procedure Save;

    procedure SaveWindowState(const Form: TForm);
    procedure RestoreWindowState(const Form: TForm);

    procedure ReadWindowPositionAndState(var Left, Top, Width, Height: integer;
                                         var WindowState: TWindowState);
    Property ApplicationTitle: String read FApplicationTitle;
  Public
    PanelHeight: Integer;
    FlatPanelWidth: Integer;
    FontSize: Integer;
    FontName: String;

    ProxyEnabled: Boolean;
    ProxyHost: String;
    ProxyPort: String;
    ProxyUsername: String;
    ProxyPassword: String;

    StartHidden: Boolean;
    PlaySounds: Boolean;
    ShowNewsFeedImages: Boolean;
    OpenLinksInNewBrowser: Boolean;
    Searchbarvisible: Boolean;
    Smartviewsexpanded: Boolean;

    LVColumnWidth1: Integer;
    LVColumnWidth2: Integer;
    LVColumnWidth3: Integer;
    VD_ItemIndex: Integer;

    ShowHeadlinesinFolder: Boolean;
    ShowNewHeadlinesinFolder: Boolean;
    ShowFeedHints: Boolean;
    ShowXmlErrors: Boolean;
    ShowAllTags: Boolean;
    ShowTopics: Boolean;
    ShowBaloons: Boolean;
    EnableTaskbarAnimations: Boolean;
    Savefilesperiodically: Boolean;
    EnableWebView: Boolean;
    OpenOneFolder: Boolean;
    WebViewPort: String;
    ToolbarShowSmallIcons: Boolean;
    EnableUserlandHttpServer: Boolean;
    EnableAmphetaDeskHttpServer: Boolean;

    CheckMsgsOnStartup: Boolean;
    AutoSearchEnabled: Boolean;
    EnableCustomsounds: Boolean;
    SoundFile:string;
    EnableJavaScript: Boolean;
    AdvancedOptions: Boolean;

    ShowTime: Boolean;
    ShowDate: Boolean;
    Fade: Boolean;
    ShowTreeLines: Boolean;
    Language: String;
    ActionCount: Integer;
    DefaultFeedRefresh: Integer;
    DefaultArchiveSize: Integer;
    UserAgent: String;
    ShowPopupsMode: TShowPopupsMode;
    MinimizeOnCloseButton: Boolean;
    ChangeUserAgent: boolean;
    ForceNewBrowserWindow: boolean;
    KeepInForeground: boolean;
    Pos_WindowLeft: integer;
    Pos_WindowTop: integer;
    Pos_WindowWidth: integer;
    Pos_WindowHeight: integer;
    Pos_PanelWidth: integer;
    Pos_ListviewHeight: integer;

    template_page:string;
    template_menu_row:string;
    template_article_row:string;

    AutoJumpToNextUnreadNextFeed: boolean; // if false show confirmation window
  Published
    Property DataDirectory: String read GetDataDirectory;
    Property RunAtStartup: Boolean read GetRunAtStartup write SetRunAtStartup;
    Property UseIEProxySettings: Boolean read FUseIEProxySettings write SetUseIEProxySettings;
    Property ApplicationDirectory: string read FApplicationDirectory write SetApplicationDirectory;
    Property AppVersion: string read GetAppVersion;
  End;

Var
  gProperties: TFeedReaderProperties = Nil;

Const
  gfIsAlphaVersion = false;

implementation

Uses
  Windows, shlobj, ShellAPI,
  SysUtils,
  Registry,
  IniFiles,
  XFunctions,
  TypInfo, Misc;

Const
  sStartupRegKey = 'Software\Microsoft\Windows\CurrentVersion\Run';

type
  TRunOnStartupAction = (rsRunOnce, rsRunAllways, rsRunNever);

//Set program to run on startup
procedure SetRunOnStartup(sProgTitle, sCmdLine: string; RunType: TRunOnStartupAction);
var
  reg: TRegIniFile;
begin
  reg := TRegIniFile.Create('');
  reg.RootKey := HKEY_CURRENT_USER;
  if RunType = rsRunOnce then
    reg.WriteString(sStartupRegKey + 'Once'#0, sProgTitle, sCmdLine)
  else if RunType = rsRunAllways then
    reg.WriteString(sStartupRegKey + #0, sProgTitle, sCmdLine)
  else
    reg.DeleteKey(sStartupRegKey + #0, sProgTitle);
  reg.Free;
end;


//Get information if program is starting automatically on windows startup
function GetRunOnStartup(sProgTitle: string): TRunOnStartupAction;
var
  reg: TRegIniFile;
begin
  reg := TRegIniFile.Create('');
  reg.RootKey := HKEY_CURRENT_USER;
  if reg.ReadString(sStartupRegKey + 'Once'#0, sProgTitle, '') <> '' then
    result := rsRunOnce
  else if reg.ReadString(sStartupRegKey + #0, sProgTitle, '') <> '' then
    result := rsRunAllways
  else
    result := rsRunNever;
  reg.Free;
end;


//Get proxy settings from Internet Explorer
function GetProxySettingsFromIE(var Host: string; var Port: integer; var ProxyEnabled: boolean): boolean;
var
  s, sporttmp: string;
  p: integer;
begin
  with TRegistry.Create do
  begin
    RootKey := HKEY_CURRENT_USER;
    ProxyEnabled := false;
    s := '';
    OpenKey('\Software\Microsoft\Windows\CurrentVersion\Internet Settings',
      True);
    if ValueExists('ProxyServer') then
      s := ReadString('ProxyServer');

    if s <> '' then
    begin
      p := pos(':', s);
      if p = 0 then
        p := length(s) + 1;
      Host := copy(s, 1, p - 1);
      try
        Port := StrToInt(copy(s, p + 1, 999));
      except
        Port := 80;
      end;

      ProxyEnabled := true;
    end;

    if ValueExists('ProxyEnable') then
    begin
      case GetDataType('ProxyEnable') of
        rdString,
          rdExpandString:
          begin
            sPortTmp := AnsiLowerCase(ReadString('ProxyEnable'));
            ProxyEnabled := true;
            if pos(' ' + sPortTmp + ' ', ' yes true t enabled 1 ') > 0 then
              ProxyEnabled := true
            else if pos(' ' + sPortTmp + ' ', ' no false f none disabled 0 ') > 0
              then
              ProxyEnabled := false
          end;
        rdInteger:
          begin
            ProxyEnabled := ReadBool('ProxyEnable');
          end;
        rdBinary:
          begin
            ProxyEnabled := true;
            ReadBinaryData('ProxyEnable', ProxyEnabled, 1);
          end;
      end;
    end;

    Free;
  end;

  Result := s <> '';
end;


Constructor TFeedReaderProperties.Create;
Begin
  Inherited Create;
  FApplicationTitle := ProgramName;
End;


Destructor TFeedReaderProperties.Destroy;
Begin
  Inherited Destroy;
End;

{-----------------------------------------------------------------------------
  Procedure: TFeedReaderProperties.GetDataDirectory
  Purpose  : Returns the directory which Feedreader uses for its files.
  Arguments: None
  Result   : String
  History  :  
-----------------------------------------------------------------------------}
Function TFeedReaderProperties.GetDataDirectory: String;
var
  pIdl: pItemIDList;
  Path: PAnsiChar;
begin
  if FileExists(IncludeTrailingPathDelimiter(FApplicationDirectory) + PropertiesFileName) then
    result := FApplicationDirectory
  else
  begin
    SHGetSpecialFolderLocation(0, CSIDL_APPDATA, pidl);
    Path := StrAlloc(MAX_PATH);
    if SHGetPathFromIDList(PIDL, Path) then
      Result := path
    Else
      Result := ExtractFilePath( ParamStr(0) );
    StrDispose(Path);
    Result := IncludeTrailingPathDelimiter( Result ) +  ProgramName;
  end;
End;

{-----------------------------------------------------------------------------
  Procedure: TFeedReaderProperties.GetRunAtStartup
  Purpose  : Returns if Feedreader runs at windows start up.
  Arguments: None
  Result   : Boolean
  History  :
-----------------------------------------------------------------------------}
Function TFeedReaderProperties.GetRunAtStartup: Boolean;
Begin
   Result := (getrunonstartup('feedreader.exe') = rsRunAllways);
End;

{-----------------------------------------------------------------------------
  Procedure: TFeedReaderProperties.SetRunAtStartup
  Purpose  : En-/Disable Feedreader start at windows start.

  Arguments: Value: Boolean
  Result   : None
  History  :
-----------------------------------------------------------------------------}
Procedure TFeedReaderProperties.SetRunAtStartup( Value: Boolean );
Begin
  If Value Then
    setrunonstartup('feedreader.exe', ParamStr(0), rsRunAllways)
  else
    setrunonstartup('feedreader.exe', ParamStr(0), rsRunnever);
End;


Procedure TFeedReaderProperties.SetUseIEProxySettings( Value: Boolean );
Var
  Port: Integer;
begin
  If FUseIEProxySettings = Value Then
    Exit;

  FUseIEProxySettings := Value;

  GetProxySettingsFromIE( ProxyHost, Port, ProxyEnabled );
  ProxyPort := IntToStr(Port);
End;

{-----------------------------------------------------------------------------
  Procedure: TFeedReaderProperties.SetApplicationDirectory
  Purpose  : Stores the application directory in the properties object.
    Then loads the stored properties.
  Author   : khoffrath
  Arguments: const Value: string
  Result   : None
  History  :
    22.11.2003 20:26:36
-----------------------------------------------------------------------------}
procedure TFeedReaderProperties.SetApplicationDirectory(const Value: string);
begin
  FApplicationDirectory := IncludeTrailingPathDelimiter(Value);
  Load;
end;

{-----------------------------------------------------------------------------
  Procedure: TFeedReaderProperties.Load
  Purpose  : Loads the saved properties from the inifile.
  Author   :
  Arguments: None
  Result   : None
  History  :
    22.11.2003 21:28:38  khoffrath
      Load and decrypt the encrypted proxy password.
-----------------------------------------------------------------------------}
Procedure TFeedReaderProperties.Load;
var
  IniFile: TIniFile;
Begin
  IniFile := TIniFile.Create(IncludeTrailingPathDelimiter(GetDataDirectory)
    + PropertiesFileName);
  PanelHeight := IniFile.ReadInteger('Properties', 'vert', 200);
  FlatPanelWidth := IniFile.ReadInteger('Properties', 'hor', 209);
  FontSize := IniFile.ReadInteger('Properties', 'fontsize', 9);
  FontName := IniFile.ReadString('Properties', 'fontname', 'Tahoma');
  ProxyEnabled := IniFile.ReadBool('Properties', 'EnableProxy', false);
  OpenOneFolder := IniFile.ReadBool('Properties', 'OpenOneFolder', false);
  SearchBarVisible := IniFile.ReadBool('Properties', 'SearchBarVisible', false);
  ProxyHost := IniFile.ReadString('Properties', 'Proxy', '');
  ProxyPort := IniFile.ReadString('Properties', 'ProxyPort', '');
  ProxyUsername := IniFile.ReadString('Properties', 'ProxyUsername', '');
  ProxyPassword := IniFile.ReadString('Properties', 'ProxyPassword', '');
  if (ProxyPassword <> '') then
  begin
    ProxyPassword := XorDecode(ProxyUsername + ProxyHost + ProxyPort,
                               ProxyPassword);
  end;
  template_page:='<html><head><title>[ACTIVEFEEDNAME]</title></head>'+
    '<body bgcolor="#ffffff" text="#000000" link="#0000ff" vlink="#000070" alink="#d1afff"> '+
    'chr(13)<table width=100%>'+
    '<tr>chr(13)<td bgcolor=#FF7F00  colspan=2>Feedreader WebAccess</tD></tr>'+
    'chr(13)<tr>chr(13)<td bgcolor=#1F558B valign=top>[CHANNELLIST]</td>chr(13)<td bgcolor=#ECEBE3 >chr(13)<table>chr(13)[ARTICLELIST]chr(13)</table>chr(13)</td>chr(13)</tr>chr(13)</table></body></html>';
  template_page := stringreplace(IniFile.ReadString('Properties', 'template_page', template_page),'chr(13)',chr(13)+chr(10),[rfreplaceall]);
  template_menu_row := stringreplace(IniFile.ReadString('Properties', 'template_menu_row', '<a href=[FEEDLOCATION]>[FEEDNAME]</a><br>'),'chr(13)',chr(13)+chr(10),[rfreplaceall]);
  template_article_row := stringreplace(IniFile.ReadString('Properties', 'template_article_row', '<tr><td><a href=[ARTICLELINK]>[ARTICLETITLE]</a></td></tr>chr(13)<tr><td>[ARTICLEBODY]<hr></td></tr>'),'chr(13)',chr(13)+chr(10),[rfreplaceall]);

  StartHidden := IniFile.ReadBool('Properties', 'start_hidden', false);
  PlaySounds := IniFile.ReadBool('Properties', 'sounds', true);
  SmartViewsExpanded := IniFile.ReadBool('Properties', 'SmartViewsExpanded', true);
  ShowNewsFeedImages := IniFile.ReadBool('Properties', 'show_images', true);
  OpenLinksInNewBrowser := IniFile.ReadBool('Properties', 'ext_links', false);

  LVColumnWidth1 := IniFile.ReadInteger('Properties', 'column1', 72);
  LVColumnWidth2 := IniFile.ReadInteger('Properties', 'column2', 371);
  LVColumnWidth3 := IniFile.ReadInteger('Properties', 'column3', 72);
  VD_ItemIndex := IniFile.ReadInteger('Properties', 'vd', 1);

  EnableCustomSounds := IniFile.ReadBool('Properties', 'EnableCustomSounds', false );
  Soundfile := IniFile.ReadString('Properties', 'Soundfile', '');

  ShowFeedHints := IniFile.ReadBool('Properties', 'ShowFeedHints', false );
  ShowXmlErrors := IniFile.ReadBool('Properties', 'show_xml_errors', false);
  ShowAllTags := IniFile.ReadBool('Properties', 'show_all_tags', true);
  ShowTopics := IniFile.ReadBool('Properties', 'show_topics', true);
  ShowBaloons := IniFile.ReadBool('Properties', 'showbaloon', true);
  AutoSearchEnabled := IniFile.ReadBool('Properties', 'AutoSearchEnabled', true);
  EnableTaskbarAnimations := IniFile.ReadBool('Properties', 'enabletaskbaranimations', true);
  SaveFilesPeriodically := IniFile.ReadBool('Properties', 'savefilesperiodically', true);
  AdvancedOptions := IniFile.ReadBool('Properties', 'AdvancedOptions', false);
  MinimizeOnCloseButton := IniFile.ReadBool('Properties', 'MinimizeOnCloseButton', true);
  ToolbarShowSmallIcons := IniFile.ReadBool('Properties', 'ToolbarShowSmallIcons', false);
  WebViewPort := IniFile.ReadString('Properties', 'WebViewPort', '80');
  EnableWebView := IniFile.ReadBool('Properties', 'enablewebserver', false);
  EnableUserlandHttpServer := IniFile.ReadBool('Properties', 'enableUserland', true);
  EnableAmphetaDeskHttpServer := IniFile.ReadBool('Properties', 'enableAmphetaDeskWebServer', true);

  CheckMsgsOnStartup := IniFile.ReadBool('Properties', 'check_on_startup', false);

  EnableJavaScript := IniFile.ReadBool('Properties', 'enablejavascript', false);
  ShowHeadlinesinFolder := IniFile.ReadBool('Properties', 'ShowHeadlinesinFolder', false);
  ShowNewHeadlinesinFolder := IniFile.ReadBool('Properties', 'ShowNewHeadlinesinFolder', true);
  ShowTime := IniFile.ReadBool('Properties', 'showtime', true);
  ShowDate := IniFile.ReadBool('Properties', 'showdate', true);
  Fade := IniFile.ReadBool('Properties', 'fade', true);
  ShowTreeLines := IniFile.ReadBool('Properties', 'treelines', true);

  Language := IniFile.ReadString('Properties', 'Language', 'english');
  ActionCount := IniFile.ReadInteger('Properties', 'Actioncount', 0);
  DefaultFeedRefresh := IniFile.ReadInteger('Properties', 'DefaultFeedRefresh', 12);
  DefaultArchiveSize := IniFile.ReadInteger('Properties', 'DefaultArchiveSize', 0);
  ChangeUserAgent := IniFile.ReadBool('Properties', 'ChangeUserAgent', true);
  if ChangeUserAgent then
    UserAgent := IniFile.ReadString('Properties', 'UserAgent', gUserAgents[0])
  else
    UserAgent := ReadIEUserAgent;
  ShowPopupsMode := TShowPopupsMode( IniFile.ReadInteger('Properties', 'ShowPopupsMode', Ord(POP_Ask)) );

  SetUseIEProxySettings( IniFile.ReadBool('Properties', 'automatic_proxy', false) );

  ForceNewBrowserWindow := IniFile.ReadBool('Properties', 'ForceNewBrowserWindow', true);
  KeepInForeground := IniFile.ReadBool('Properties', 'KeepInForeground', true);
  Pos_PanelWidth := IniFile.ReadInteger('Properties', 'Pos_PanelWidth', -1);
  Pos_ListviewHeight := IniFile.ReadInteger('Properties', 'Pos_ListviewHeight', -1);
  AutoJumpToNextUnreadNextFeed := IniFile.ReadBool('Properties', 'AutoJumpNextUnreadNextFeed', true);
  IniFile.Free;
End;

{-----------------------------------------------------------------------------
  Procedure: TFeedReaderProperties.Save
  Purpose  :
  Author   :
  Arguments: None
  Result   : None
  History  :
    22.11.2003 21:30:05  khoffrath
      Encrypt the proxy password and save it in the inifile.
-----------------------------------------------------------------------------}
Procedure TFeedReaderProperties.Save;
var
  IniFile: TIniFile;
Begin
  IniFile := TIniFile.Create(IncludeTrailingPathDelimiter(GetDataDirectory)
    + PropertiesFileName);
  IniFile.WriteInteger('Properties', 'vert', PanelHeight);
  IniFile.WriteInteger('Properties', 'hor', FlatPanelWidth);
  IniFile.WriteInteger('Properties', 'fontsize', FontSize);
  IniFile.WriteString('Properties', 'fontname', FontName );
  IniFile.WriteBool('Properties', 'EnableProxy', ProxyEnabled);
  IniFile.WriteString('Properties', 'Proxy', ProxyHost);
  IniFile.WriteString('Properties', 'ProxyPort', ProxyPort);
  IniFile.WriteString('Properties', 'ProxyUsername', ProxyUsername);
  IniFile.WriteString('Properties', 'ProxyPassword',
                      XorEncode(ProxyUsername + ProxyHost + ProxyPort,
                                ProxyPassword));
  IniFile.WriteBool('Properties', 'start_hidden', StartHidden);
  IniFile.WriteBool('Properties', 'sounds', PlaySounds);
  IniFile.WriteBool('Properties', 'show_images', ShowNewsFeedImages);
  IniFile.WriteBool('Properties', 'ext_links', OpenLinksInNewBrowser);
  IniFile.WriteBool('Properties', 'automatic_proxy', UseIEProxySettings);
  IniFile.WriteBool('Properties', 'OpenOneFolder', OpenOneFolder);
  IniFile.WriteBool('Properties', 'SmartViewsExpanded', SmartViewsExpanded);

  IniFile.WriteInteger('Properties', 'column1', LVColumnWidth1);
  IniFile.WriteInteger('Properties', 'column2', LVColumnWidth2);
  IniFile.WriteInteger('Properties', 'column3', LVColumnWidth3);
  IniFile.WriteInteger('Properties', 'vd', VD_ItemIndex);

  IniFile.WriteBool('Properties', 'EnableCustomSounds', EnableCustomSounds);
  IniFile.WriteString('Properties', 'Soundfile', Soundfile);

  IniFile.WriteBool('Properties', 'SearchBarVisible', Searchbarvisible);
  IniFile.WriteBool('Properties', 'ShowFeedHints', ShowFeedHints);
  IniFile.WriteBool('Properties', 'show_xml_errors', ShowXmlErrors);
  IniFile.WriteBool('Properties', 'show_all_tags', ShowAllTags);
  IniFile.WriteBool('Properties', 'show_topics', ShowTopics);
  IniFile.WriteBool('Properties', 'showbaloon', ShowBaloons);
  IniFile.WriteBool('Properties', 'enabletaskbaranimations', EnableTaskbarAnimations);
  IniFile.WriteBool('Properties', 'savefilesperiodically', SaveFilesPeriodically);
  IniFile.WriteBool('Properties', 'AdvancedOptions', AdvancedOptions);
  IniFile.WriteBool('Properties', 'ToolbarShowSmallIcons', ToolbarShowSmallIcons);
  IniFile.WriteBool('Properties', 'AutoSearchEnabled', AutoSearchEnabled);

  IniFile.WriteBool('Properties', 'MinimizeOnCloseButton', MinimizeOnCloseButton );

  IniFile.WriteBool('Properties', 'enablewebserver', EnableWebView );
  IniFile.WriteString('Properties', 'WebViewPort', WebViewPort);

  IniFile.WriteBool('Properties', 'enableUserland', EnableUserlandHttpServer);
  IniFile.WriteBool('Properties', 'enableAmphetaDeskWebServer', EnableAmphetaDeskHttpServer);

  IniFile.WriteBool('Properties', 'check_on_startup', CheckMsgsOnStartup);

  IniFile.WriteBool('Properties', 'enablejavascript', EnableJavaScript);

  IniFile.WriteBool('Properties', 'showtime', ShowTime);
  IniFile.WriteBool('Properties', 'showdate', ShowDate);
  IniFile.WriteBool('Properties', 'fade', Fade);
  IniFile.WriteBool('Properties', 'treelines', ShowTreeLines);
  IniFile.WriteBool('Properties', 'ShowHeadlinesinFolder', ShowHeadlinesinFolder);
  IniFile.WriteBool('Properties', 'ShowNewHeadlinesinFolder', ShowNewHeadlinesinFolder);

  IniFile.WriteString('Properties', 'Language', Language);
  IniFile.WriteInteger('Properties', 'Actioncount', ActionCount);
  IniFile.WriteInteger('Properties', 'DefaultFeedRefresh', DefaultFeedRefresh);
  IniFile.WriteInteger('Properties', 'DefaultArchiveSize', DefaultArchiveSize);
  IniFile.WriteBool('Properties', 'ChangeUserAgent', ChangeUserAgent);
  IniFile.WriteString('Properties', 'UserAgent', UserAgent );

  IniFile.WriteString('Properties', 'template_page', stringreplace(template_page,chr(13)+chr(10),'chr(13)',[rfreplaceall]) );
  IniFile.WriteString('Properties', 'template_menu_row', stringreplace(template_menu_row,chr(13)+chr(10),'chr(13)',[rfreplaceall]) );
  IniFile.WriteString('Properties', 'template_article_row', stringreplace(template_article_row,chr(13)+chr(10),'chr(13)',[rfreplaceall]) );

  IniFile.WriteInteger('Properties', 'ShowPopupsMode', Ord(ShowPopupsMode) );
  IniFile.WriteBool('Properties', 'ForceNewBrowserWindow', ForceNewBrowserWindow);
  IniFile.WriteBool('Properties', 'KeepInForeground', KeepInForeground);

  IniFile.WriteInteger('Properties', 'Pos_PanelWidth', Pos_PanelWidth);
  IniFile.WriteInteger('Properties', 'Pos_ListviewHeight', Pos_ListviewHeight);
  IniFile.WriteBool('Properties', 'AutoJumpNextUnreadNextFeed', AutoJumpToNextUnreadNextFeed);

  IniFile.Free;
End;

{-----------------------------------------------------------------------------
  Procedure: TFeedReaderProperties.RestoreWindowState
  Purpose  : Restore the windowstate and position  of the given window.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: const Form: TForm
  Result   : None
  History  :
    14.12.2003 21:18:11
-----------------------------------------------------------------------------}
procedure TFeedReaderProperties.RestoreWindowState(const Form: TForm);
var
 L, T, W, H: Integer;
 Ini: TIniFile;
 SavedWindowState: TWindowState;
 PreviousVisible: boolean;
begin
  Ini := TIniFile.Create(IncludeTrailingPathDelimiter(GetDataDirectory)
    + PropertiesFileName);
  L := Ini.ReadInteger('Properties', 'Pos_WindowLeft', 195);
  T := Ini.ReadInteger('Properties', 'Pos_WindowTop', 140);
  W := Ini.ReadInteger('Properties', 'Pos_WindowWidth', 680);
  H := Ini.ReadInteger('Properties', 'Pos_WindowHeight', 480);

  try
    SavedWindowState :=
      TWindowState(GetEnumValue(TypeInfo(TWindowState),
                   Ini.ReadString('Properties', 'Pos_WindowState', 'wsNormal')));
  except
    SavedWindowState := wsNormal;
  end;
// Build 622
  PreviousVisible := Form.Visible;
  Form.Visible := true;   
  case SavedWindowState of
    wsMinimized:
      begin
        if (Form.WindowState <> wsNormal) then Form.WindowState := wsNormal;
        Form.SetBounds(L, T, W, H);
        Form.WindowState := wsMinimized;
      end;
    wsNormal: Form.SetBounds(L, T, W, H);
    wsMaximized:
      begin
        if (Form.WindowState <> wsNormal) then Form.WindowState := wsNormal;
        Form.SetBounds(L, T, W, H);
        Form.WindowState := wsMaximized;
      end;
  end;
  Form.Visible := PreviousVisible;
  FreeAndNil(Ini);
end;

{-----------------------------------------------------------------------------
  Procedure: TFeedReaderProperties.SaveWindowState
  Purpose  : Save the windowstate and position of the given window.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: const Form: TForm
  Result   : None
  History  :
    14.12.2003 21:13:18
-----------------------------------------------------------------------------}
procedure TFeedReaderProperties.SaveWindowState(const Form: TForm);
var
  WP: TWindowPlacement;
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(IncludeTrailingPathDelimiter(GetDataDirectory)
    + PropertiesFileName);
  if (Form.WindowState = wsNormal) then
  begin
    WP.Length := Sizeof(WP);
    GetWindowPlacement(Form.Handle, @WP);
    Ini.WriteInteger('Properties', 'Pos_WindowLeft', wp.rcNormalPosition.Left);
    Ini.WriteInteger('Properties', 'Pos_WindowTop', wp.rcNormalPosition.Top);
    Ini.WriteInteger('Properties', 'Pos_WindowWidth',
                     wp.rcNormalPosition.Right-wp.rcNormalPosition.Left);
    Ini.WriteInteger('Properties', 'Pos_WindowHeight',
                     wp.rcNormalPosition.Bottom-wp.rcNormalPosition.Top);
  end;
  Ini.WriteString('Properties', 'Pos_WindowState',
                  GetEnumName(TypeInfo(TWindowState), Ord(Form.WindowState)));
  FreeAndNil(Ini);
end;

{-----------------------------------------------------------------------------
  Procedure: TFeedReaderProperties.ReadWindowPositionAndState
  Purpose  : Reads the windowdimensions and windowstate from the inifile and
    saves them in the variables.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: var Top, Left, Width, Height: integer; var WindowState: TWindowState
  Result   : None
  History  :
    20.12.2003 19:11:43
-----------------------------------------------------------------------------}
procedure TFeedReaderProperties.ReadWindowPositionAndState(var Left, Top,
  Width, Height: integer; var WindowState: TWindowState);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(IncludeTrailingPathDelimiter(GetDataDirectory)
    + PropertiesFileName);
  Left := Ini.ReadInteger('Properties', 'Pos_WindowLeft', 195);
  Top := Ini.ReadInteger('Properties', 'Pos_WindowTop', 140);
  Width := Ini.ReadInteger('Properties', 'Pos_WindowWidth', 680);
  Height := Ini.ReadInteger('Properties', 'Pos_WindowHeight', 480);
  try
    WindowState :=
      TWindowState(GetEnumValue(TypeInfo(TWindowState),
                   Ini.ReadString('Properties', 'Pos_WindowState', 'wsNormal')));
  except
    WindowState := wsNormal;
  end;
  FreeAndNil(Ini);
end;

function TFeedReaderProperties.GetAppVersion: string;
begin
  result := Sto_GetFileVersion(Application.ExeName, 4);
end;

end.
