(*
 *   FeedReader
 *   Copyright (C) 2003 The FeedReader Developement Group
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *)
unit HTTPServers;

interface

Uses
  Windows, Classes, SysUtils, Misc, XFunctions, FReaderProperties,
  IdTCPServer, idSocketHandle, IdCustomHTTPServer, IdHTTPServer, IdHTTPHeaderInfo;


Type
  TAddNewFeedEvent = Procedure( Const URL: String ) Of Object;
  
Type
  TAbstractHttpHandler = Class( TObject )
  End;

  TWebViewHttpHandler = Class( TAbstractHttpHandler )
  private
    Owner: TIdHTTPServer;
    Procedure OnCommandGet( AThread: TIdPeerThread; RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo );
    function genhtmlchannel( SelectedFeedID: Real; FeedIndex: Integer ): string;
    function genhtmlchannellist( SelectedFeedID: Real ): string;
  public
    Constructor Create( AOwner: TIdHTTPServer );
    Destructor Destroy; override;
  End;

  TAddFeedHttpHandler = Class( TAbstractHttpHandler )
  Private
    Owner: TIdHTTPServer;
    OnAddNewFeddEventHandler: TAddNewFeedEvent;

    procedure SendResponsePage( RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo; Const Content: String );
    procedure SendAddMultipleFeedsPage( RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo; Const URLs: String );
    Procedure OnCommandGet( AThread: TIdPeerThread; RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo );
  public
    Constructor Create( AOwner: TIdHTTPServer; EventHandler: TAddNewFeedEvent );
    Destructor Destroy; override;
  End;

type
  TEncapsulateOnConnectEvent = class(TObject)
  private
  public
    procedure OnConnect(AThread: TIdPeerThread);
  end;

Procedure EnableUserlandLocalHttpServer( Owner: TComponent; Enable: Boolean; Const DataDirectory: String; EventHandler: TAddNewFeedEvent );
Procedure EnableWebViewHttpServer( Owner: TComponent; Enable: Boolean; Const DataDirectory, Port: String );
Procedure EnableAmphetaDeskHttpServer( Owner: TComponent; Enable: Boolean; Const DataDirectory: String; EventHandler: TAddNewFeedEvent );


Var
  gUserlandHttpServer: TIdHTTPServer = Nil;
  gUserlandHttpHandler: TAddFeedHttpHandler = Nil;
  gUserlandConnectHandler: TEncapsulateOnConnectEvent = nil;
  gWebViewHttpServer: TIdHTTPServer = Nil;
  gWebViewHttpHandler: TWebViewHttpHandler = Nil;
  gWebViewConnectHandler: TEncapsulateOnConnectEvent = nil;
  gAmphetaDeskHttpServer: TIdHTTPServer = Nil;
  gAmphetaDeskHttpHandler: TAddFeedHttpHandler = Nil;
  gAmphetaDeskConnectHandler: TEncapsulateOnConnectEvent = nil;


implementation

// /////////////////////////////////////////////////////////////////////////////
//
// TWebViewHttpHandler
//
// /////////////////////////////////////////////////////////////////////////////
Constructor TWebViewHttpHandler.Create( AOwner: TIdHTTPServer );
Begin
  Inherited Create;

  Owner := AOwner;
  Owner.OnCommandGet := Self.OnCommandGet;
End;


Destructor TWebViewHttpHandler.Destroy;
Begin
  Inherited Destroy;
End;


function TWebViewHttpHandler.genhtmlchannellist( SelectedFeedID: Real ): string;
var
  tmpstr,activerow: string;
  i: Integer;
begin
  try
    //Generate html contents for serving by internal webserver
    For i := Low(gSubscribedFeeds) To High(gSubscribedFeeds) Do Begin
      if (gSubscribedFeeds[i].feedid > 0) and (gSubscribedFeeds[i].allowfromweb) and (gSubscribedFeeds[i].feedtype <> 'folder') then
      begin
          activerow:=stringreplace(gproperties.template_menu_row,'[FEEDLOCATION]','/channel/'+floattostr(gSubscribedFeeds[i].feedid),[rfreplaceall]);
          activerow:=stringreplace(activerow,'[FEEDNAME]',gsubscribedfeeds[i].title,[rfreplaceall]);
          tmpstr := tmpstr + activerow;
      end;
    end;
    genhtmlchannellist := tmpstr;
  finally
  end;
end;


function TWebViewHttpHandler.genhtmlchannel( SelectedFeedID: Real; FeedIndex: Integer ): string;
//Generate html contents for serving by internal webserver
var
  MyDate, mytime: TDateTime;
  i, mitu, kuu_paevi, dy, dm, dd, dh,
  dmin, hourshift: integer;
  leitud: boolean;
  tzz, da1, da2, dat,channellist,articlelist,activearticle: string;
  tmpstr: string;
begin
  MyDate := Now;
  hourshift := 0;
  tmpstr:=gproperties.template_page;
//  EnterCriticalSection(gCritSect);
  try
    if SelectedFeedID <> 0 then
      tmpstr := stringreplace(tmpstr,'[ACTIVEFEEDNAME]',gSubscribedFeeds[FeedIndex].title,[rfreplaceall])
    else
      tmpstr := stringreplace(tmpstr,'[ACTIVEFEEDNAME]','',[rfreplaceall]);

      channellist:=genhtmlchannellist(SelectedFeedID);
      tmpstr := stringreplace(tmpstr,'[CHANNELLIST]',channellist,[rfreplaceall]);

    articlelist:='';
    if (SelectedFeedID <> 0) And (FeedIndex >= 0) Then
    begin
      leitud := false;
      mitu := 0;
      for i := gHeadlines.count - 1 downto 0 do
      begin

        if (mitu > 19) or (leitud and (theadline(gHeadlines[i]).owner <> gSubscribedFeeds[FeedIndex].feedid)
          and not theadline(gHeadlines[i]).deleted) then
          break;
        if (theadline(gHeadlines[i]).owner = gSubscribedFeeds[FeedIndex].feedid) and not theadline(gHeadlines[i]).deleted then
        begin
          inc(mitu);
          tzz := copy(theadline(gHeadlines[i]).date, 1 + pos('+', theadline(gHeadlines[i]).date), 2);
          try
            hourshift := (GetHourfromGMT * -1) - strtoint(trim(tzz));
          except
            on e: Econverterror do
            begin
              hourshift := 0;
            end;

          end;
          if (pos('-', theadline(gHeadlines[i]).date) > 0) and (length(theadline(gHeadlines[i]).date) > 11) then
          begin
            dat := copy(theadline(gHeadlines[i]).date, 12, 5);
            da1 := copy(dat, 1, 2);
            da2 := copy(dat, 4, 2);
            dmin := strtoint(da2);
            dh := strtoint(da1) + hourshift;
            dd := strtoint(copy(theadline(gHeadlines[i]).date, 9, 2));
            dm := strtoint(copy(theadline(gHeadlines[i]).date, 6, 2));
            dy := strtoint(copy(theadline(gHeadlines[i]).date, 0, 4));
            if dh >= 24 then
            begin
              dh := dh - 24;
              dd := dd + 1;
            end
            else if dh < 0 then
            begin
              dh := 24 + dh;
              dd := dd - 1;
            end;

            if (dm = 1) or (dm = 3) or (dm = 5) or (dm = 7) or (dm = 8) or (dm =
              10) or (dm = 12) then
              kuu_paevi := 31
            else if (dm = 2) then
              kuu_paevi := 28
            else
              kuu_paevi := 30;

            if dd > kuu_paevi then
            begin
              dd := dd - kuu_paevi;
              dm := dm + 1;
            end
            else if dd < 1 then
            begin
              dd := kuu_paevi - dd;
              dm := dm - 1;
            end;
            if dm > 12 then
            begin
              dm := dm - 12;
              dy := dy + 1;
            end
            else if dm < 1 then
            begin
              dm := 12 - dm;
              dy := dy - 1;
            end;

            if dh < 0 then
              dh := 24 + dh;
            try
              mydate := encodedate(dy, dm, dd);
            except
            end;
            mytime := encodetime(dh, dmin, 0, 0);

            if (gProperties.ShowDate and gProperties.ShowTime) then
              dat := formatdatetime('hh:nn dd.mm', mydate + mytime)
            else if not (gProperties.ShowDate or gProperties.ShowTime) then
              dat := formatdatetime('hh:nn dd.mm', mydate + mytime)
            else if gProperties.ShowDate and not gProperties.ShowTime then
              dat := formatdatetime('dd.mm', mydate + mytime)
            else if gProperties.ShowTime and not gProperties.ShowDate then
              dat := formatdatetime('hh:mm', mydate + mytime);
          end;

          activearticle := activearticle + dat;
          activearticle:=stringreplace(gproperties.template_article_row,'[ARTICLEDATE]',dat,[rfreplaceall]);
          if theadline(gHeadlines[i]).link <> '' then
          activearticle:=stringreplace(activearticle,'[ARTICLELINK]',theadline(gHeadlines[i]).link,[rfreplaceall])
          else
          activearticle:=stringreplace(activearticle,'[ARTICLELINK]','',[rfreplaceall]);

          activearticle:=stringreplace(activearticle,'[ARTICLETITLE]',theadline(gHeadlines[i]).tiitel,[rfreplaceall]);

          if theadline(gHeadlines[i]).sisu <> '' then
          activearticle:=stringreplace(activearticle,'[ARTICLEBODY]',theadline(gHeadlines[i]).sisu,[rfreplaceall])
          else
          activearticle:=stringreplace(activearticle,'[ARTICLEBODY]','',[rfreplaceall]);



          articlelist:=articlelist+activearticle;
        end;
      end;
    end;
    tmpstr := stringreplace(tmpstr,'[ARTICLELIST]',articlelist,[rfreplaceall]);
    genhtmlchannel := tmpstr;
  finally
//    LeaveCriticalSection(gCritSect);
  end;
end;


procedure TWebViewHttpHandler.OnCommandGet( AThread: TIdPeerThread; RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo );
Const
  ChannelPath = '/channel/';
var
  content: string;
  FeedID: real;
  i: Integer;
  FeedIndex: Integer;
  Doc: String;
begin
  content := '';
  Doc := RequestInfo.Document;
  if 0 = CompareText( ChannelPath, Copy(Doc, 1, Length(ChannelPath))) then begin
    Delete( Doc, 1, Length(ChannelPath) ); 
    FeedID := StrToFloatDef( Doc, 0);

    FeedIndex := -1;
//    EnterCriticalSection(gCritSect);
    try
      For i := Low(gSubscribedFeeds) To High(gSubscribedFeeds) Do
        if (gSubscribedFeeds[i].feedid = FeedID) and (gSubscribedFeeds[i].feedtype <> 'folder') then Begin
          FeedIndex := i;
          Break;
        End;
    finally
//      LeaveCriticalSection(gCritSect);
    end;

    content := content + genhtmlchannel( FeedID, FeedIndex );
  end
  else if RequestInfo.Document = '/' then
    content := content + genhtmlchannel( 0, -1 )
  else
    content := '<html><head><title>No access<title></head><body><h1>No Access</h1></body></html>';


  ResponseInfo.ResponseNo := 200;
  ResponseInfo.ContentText := Content;
end;


// /////////////////////////////////////////////////////////////////////////////
//
// TAddFeedHttpHandler
//
// /////////////////////////////////////////////////////////////////////////////
Constructor TAddFeedHttpHandler.Create( AOwner: TIdHTTPServer; EventHandler: TAddNewFeedEvent );
Begin
  Inherited Create;

  Owner := AOwner;
  Owner.OnCommandGet := Self.OnCommandGet;
  OnAddNewFeddEventHandler := EventHandler;
End;


Destructor TAddFeedHttpHandler.Destroy;
Begin
  Inherited Destroy;
End;


Procedure TAddFeedHttpHandler.OnCommandGet( AThread: TIdPeerThread; RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo );
var
  content, url: string;
begin
  content := '';

  if Length(RequestInfo.Params.Values['add_urls']) > 0 then begin
    url := RequestInfo.Params.Values['add_urls'];
    url := UnEscapeURL( url );

    SendAddMultipleFeedsPage( RequestInfo, ResponseInfo, url );
  end
  else if Length(RequestInfo.Params.Values['add_url']) > 0 then begin
    url := RequestInfo.Params.Values['add_url'];
    url := UnEscapeURL( url );

    If Assigned( OnAddNewFeddEventHandler ) Then
      OnAddNewFeddEventHandler( url );

    SendResponsePage( RequestInfo, ResponseInfo, '' );
  end
  else if Length(RequestInfo.Params.Values['url']) > 0 then begin
    url := RequestInfo.Params.Values['url'];
    url := UnEscapeURL( url );

    If Assigned( OnAddNewFeddEventHandler ) Then
      OnAddNewFeddEventHandler( url );

    SendResponsePage( RequestInfo, ResponseInfo, '' );
  end
   Else Begin
     content := 'No access';
     SendResponsePage( RequestInfo, ResponseInfo, Content );
   End;

End;


procedure TAddFeedHttpHandler.SendResponsePage( RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo; Const Content: String );
Var
  Body: string;
begin
  { Create a stream to hold data sent to client that is the answer }
  { made of a HTTP header and a body made of HTML code.            }
  if Length(Content) = 0 then
  begin
    Body := '<HTML><HEAD><TITLE>Feedreader</TITLE>';
    body := body + '<style type="text/css">td,body,.form {font-family:Tahoma; font-size: 9pt}A {color:#336699; text-decoration: none;}</style>';
    body := body + '</HEAD>' + #13#10 +
          '<BODY onload="javascript:history.back()"><h1>Feedreader</h1>Adding feed...<br><br><a href="javascript:history.back()">Back</a></BODY>' +
          '</HTML>' + #13#10;
  end
  else
    body := content;

  ResponseInfo.ResponseNo := 200;
  ResponseInfo.ContentText := Body;
end;


procedure TAddFeedHttpHandler.SendAddMultipleFeedsPage( RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo; Const URLs: String );
var
  Body: string;
  i: Integer;
  NewURL, Temp: String;
begin
  { made of a HTTP header and a body made of HTML code.            }
  Body := '<HTML><HEAD><TITLE>Feedreader</TITLE>';
  body := body + '<style type="text/css">td,body,.form {font-family:Tahoma; font-size: 9pt}A {color:#336699; text-decoration: none;}</style>';
  body := body + '</HEAD>' + #13#10 +
        '<BODY><h1>Feedreader</h1>';
  body := body + '<h4>Click on the links below to add the feed</h4>';

  Temp := URLs;
  i := Pos(',', Temp);
  While i > 0 Do Begin
    NewURL := Trim( Copy( Temp, 1, i-1 ) );

    If Length(NewURL) > 0 Then
      body := body + '<A HREF="http://127.0.0.1:' + IntToStr(Owner.Bindings[0].Port) + '/?add_url=' + EscapeURL(NewURL) + '">' + EscapeHTML(NewURL) + '</A><BR>';

    Temp := Copy( Temp, i+1, Length(Temp) );
    i := Pos(',', Temp);
  End; // while

  body := body + '<br><br><a href="javascript:history.back()">Back</a></BODY>' +
          '</HTML>' + #13#10;

  SendResponsePage( RequestInfo, ResponseInfo, Body );
end;


//
//
// EnableUserlandLocalHttpServer
//
//
Procedure EnableUserlandLocalHttpServer( Owner: TComponent; Enable: Boolean; Const DataDirectory: String; EventHandler: TAddNewFeedEvent );
Var
  Binding : TIdSocketHandle;
Begin
  If Not Enable Then
  Begin
    If Assigned( gUserlandHttpServer ) Then
      gUserlandHttpServer.Active := False;

    FreeAndNil( gUserlandHttpServer );
    FreeAndNil( gUserlandHttpHandler );
    FreeAndNil( gUserlandConnectHandler );
    Exit;
  End;

  If Enable And (Not Assigned( gUserlandHttpServer )) Then
    Try
      gUserlandHttpServer := TIdHTTPServer.Create( Owner );
      gUserlandHttpServer.Bindings.Clear;
      gUserlandConnectHandler := TEncapsulateOnConnectEvent.Create;
      gUserlandHttpServer.OnConnect := gUserlandConnectHandler.OnConnect;
      Binding := gUserlandHttpServer.Bindings.Add;
      Binding.Port := 5335;
      Binding.IP := '127.0.0.1';

      gUserlandHttpHandler := TAddFeedHttpHandler.Create( gUserlandHttpServer, EventHandler );

      gUserlandHttpServer.Active := True;
    Except
      FreeAndNil( gUserlandHttpServer );
      FreeAndNil( gUserlandHttpHandler );
      FreeAndNil( gUserlandConnectHandler );
    End;
End;


//
//
// EnableWebViewHttpServer
//
//
Procedure EnableWebViewHttpServer( Owner: TComponent; Enable: Boolean; Const DataDirectory, Port: String );
Var
  Binding : TIdSocketHandle;
Begin
  If Not Enable Then
  Begin
    If Assigned( gWebViewHttpServer ) Then
      gWebViewHttpServer.Active := False;

    FreeAndNil( gWebViewHttpServer );
    FreeAndNil( gWebViewHttpHandler );
    FreeAndNil( gWebViewConnectHandler );
    Exit;
  End;

  If Enable And (Not Assigned( gWebViewHttpServer )) Then
    Try
      gWebViewHttpServer := TIdHttpServer.Create( Owner );
      gWebViewHttpServer.Bindings.Clear;
      gWebViewConnectHandler := TEncapsulateOnConnectEvent.Create;
      gWebViewHttpServer.OnConnect := gWebViewConnectHandler.OnConnect;
      Binding := gWebViewHttpServer.Bindings.Add;
      Binding.Port := StrToIntDef( Port, 80 );
      //Binding.IP := '127.0.0.1';

      gWebViewHttpHandler := TWebViewHttpHandler.Create( gWebViewHttpServer );

      gWebViewHttpServer.Active := True;
    Except
      FreeAndNil( gWebViewHttpServer );
      FreeAndNil( gWebViewHttpServer );
      FreeAndNil( gWebViewConnectHandler );
    End;
End;


//
//
// EnableAmphetaDeskHttpServer
//
//
Procedure EnableAmphetaDeskHttpServer( Owner: TComponent; Enable: Boolean; Const DataDirectory: String; EventHandler: TAddNewFeedEvent );
Var
  Binding : TIdSocketHandle;
Begin
  If Not Enable Then
  Begin
    If Assigned( gAmphetaDeskHttpServer ) Then
      gAmphetaDeskHttpServer.Active := False;

    FreeAndNil( gAmphetaDeskHttpServer );
    FreeAndNil( gAmphetaDeskHttpHandler );
    FreeAndNil( gAmphetaDeskConnectHandler );
    Exit;
  End;

  If Enable And (Not Assigned( gAmphetaDeskHttpServer )) Then
    Try
      gAmphetaDeskHttpServer := TIdHTTPServer.Create( Owner );
      gAmphetaDeskConnectHandler := TEncapsulateOnConnectEvent.Create;
      gAmphetaDeskHttpServer.OnConnect := gAmphetaDeskConnectHandler.OnConnect;
      gAmphetaDeskHttpServer.Bindings.Clear;
      Binding := gAmphetaDeskHttpServer.Bindings.Add;
      Binding.Port := 8888;
      Binding.IP := '127.0.0.1';

      gAmphetaDeskHttpHandler := TAddFeedHttpHandler.Create( gAmphetaDeskHttpServer, EventHandler );

      gAmphetaDeskHttpServer.Active := True;
    Except
      FreeAndNil( gAmphetaDeskHttpServer );
      FreeAndNil( gAmphetaDeskHttpHandler );
      FreeAndNil( gAmphetaDeskConnectHandler );
    End;
End;

{ TEncapsulateOnConnectEvent }

procedure TEncapsulateOnConnectEvent.OnConnect(AThread: TIdPeerThread);
begin
  {
    Allow only connections from the local computer.
  }
//  if (AThread.Connection.Socket.Binding.PeerIP <> '127.0.0.1') then
//    AThread.Connection.Disconnect;
end;

end.
