(*
 *   FeedReader
 *   Copyright (C) 2003 The FeedReader Developement Group
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *)
unit LanguageSupport;

interface

uses
  Classes, Forms, Menus, ActnMan, ActnCtrls, IniFiles,StdCtrls,ComCtrls,extctrls;

Procedure LoadTranslations( Const Filename: String );
Function TranslateString( Const EnglishText: String ): String;
procedure TranslateAllForms;

procedure TranslateForm(const Form: TForm);
procedure TranslateActionBarItem(const ActionBarItem: TActionBarItem);
procedure TranslateActionItem(const ActionItem: TActionClientItem);

Function GetUsedLanguage: String;

implementation

Uses
  Controls, ActnList,
  XDOM_2_3,
  XFunctions,
  SysUtils;


Type
  TCaptionAccess = class(TControl);

  TTranslation = record
    english, translated: string;
  end;

Var
  gTranslations: array Of TTranslation;
  gCurLanguage: String = 'english';

{
  To create a list of all strings that nust be translated, set the filename to
  use, enable this define and build FeedReader.
  Then run FeedReader and open every form, tab, etc.
}
{.define WRITETRANSLATESTRINGS;
}
Function TranslateString( Const EnglishText: String ): String;
Var
  i: Integer;
  {$ifdef WRITETRANSLATESTRINGS}
  OutputFile: TStringList;
  {$endif}
Begin
  {$ifdef WRITETRANSLATESTRINGS}
  if (Trim(EnglishText) <> '') then
  begin
    OutputFile := TStringList.Create;
    try
      try
        {
          Catch exception if file doesn't exists.
        }
        OutputFile.LoadFromFile('c:\Translatestrings.txt');
      except
      end;
      if (OutputFile.IndexOf(EnglishText) = -1) then
      begin
        OutputFile.Add(EnglishText);
        OutputFile.SaveToFile('c:\Translatestrings.txt');
      end;
    finally
      FreeAndNil(OutputFile);
    end;
  end;
  {$endif}
  For i := 0 to length(gTranslations) - 1 do
    if EnglishText = gTranslations[i].english then
      if gTranslations[i].translated <> '' then Begin
        Result := gTranslations[i].translated;
        Exit;
      End;

  Result := EnglishText;
End;

procedure TranslateAllForms;
Var
  iForm,i: Integer;
  iComponent: Integer;
  CurForm: TForm;
  CurComponent: TComponent;
  iActionToolbar: integer;
begin
  //This functions translates all form components
  for iForm := 0 to application.ComponentCount - 1 do
  Begin
    if Not (application.Components[iForm] is TForm) then
      Continue;

    CurForm := Application.Components[iForm] As TForm;

    CurForm.Caption := TranslateString( CurForm.Caption );

    for iComponent := 0 to CurForm.componentcount - 1 do
    Begin
      if Not (CurForm.Components[iComponent] is TComponent) then
        Continue;

      CurComponent := CurForm.Components[iComponent] As TComponent;

      // translate caption-properties
      //
      if CurComponent Is TControl Then
        TCaptionAccess(CurComponent).Caption := TranslateString( TCaptionAccess(CurComponent).Caption );

      if Curcomponent is TComboboxEx then
      for i:=0 to TComboboxEx(CurComponent).itemsex.Count-1 do
      begin
      TComboboxEx(CurComponent).itemsex[i].caption:=translatestring(TComboboxEx(CurComponent).itemsex[i].caption);
      end;


      // translate actions
      //
      if CurComponent Is TAction Then Begin

        // caption
        TAction(CurComponent).Caption := TranslateString( TAction(CurComponent).Caption );

        // hint
        TAction(CurComponent).Hint := TranslateString( TAction(CurComponent).Hint );
      End;

      if CurComponent is TMenuItem then
      begin
        TMenuItem(CurComponent).Caption := TranslateString(TMenuItem(CurComponent).Caption);
        TMenuItem(CurComponent).Hint := TranslateString(TMenuItem(CurComponent).Hint);
      end;

      {
        If the current component is a TActionManager then call for each
        associated ActionBar the method TranslateActionToolBar.
      }
      if CurComponent is TActionManager then
      begin
        if (TActionManager(CurComponent).ActionBars.Count > 0) then
        begin
          for iActionToolbar := 0 to TActionManager(CurComponent).ActionBars.Count-1 do
          begin
            TranslateActionBarItem(TActionManager(CurComponent).ActionBars[iActionToolbar]);
          end;
        end;
      end;
      
    End; // for iComponent
  End; //for iForm

  (*
  //MH: TODO
  listview1.Columns[1].Caption := t('Date');
  listview1.Columns[2].Caption := t('Headline');
  if actionmanager1.ActionBars[0].Items[0].caption <>
    t(actionmanager1.ActionBars[0].Items[0].caption) then
    donotsave := true;
  *)
end;


Procedure LoadTranslations( Const Filename: String );
var
  i: integer;
  IniFile: TIniFile;
  TempString: string;
  Texts: TStringList;
begin
  IniFile := TIniFile.Create(Filename);
  Texts := TStringList.Create;
  try
    IniFile.ReadSectionValues('Texts', Texts);
    for i := 0 to Texts.Count-1 do
    begin
      TempString := Texts[i];
      while (Pos('"', TempString) <> 0) do
        Delete(TempString, Pos('"', TempString), 1);
      Texts[i] := TempString;
    end;
    SetLength(gTranslations, Texts.Count);
    for i := 0 to Texts.Count-1 do
    begin
      gTranslations[i].english := Texts.Names[i];
      gTranslations[i].translated := Texts.Values[Texts.Names[i]];
    end;
  finally
    FreeAndNil(Texts);
    FreeAndNil(IniFile);
  end;
End;


Function GetUsedLanguage: String;
Begin
  Result := gCurLanguage;
End;

{-----------------------------------------------------------------------------
  Procedure: TranslateForm
  Purpose  : Translate the given form and all of its subcomponents.
    Functionality similar to TranslateAllForms.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: const Form: TForm
  Result   : None
  History  :
    07.12.2003 16:15:12
-----------------------------------------------------------------------------}
procedure TranslateForm(const Form: TForm);
var
  iComponent,i: Integer;
  CurComponent: TComponent;
begin
  Form.Caption := TranslateString(Form.Caption);
  for iComponent := 0 to Form.ComponentCount-1 do
  Begin
    if not (Form.Components[iComponent] is TComponent) then
      Continue;

    CurComponent := Form.Components[iComponent] as TComponent;

    if CurComponent is TControl then
      TCaptionAccess(CurComponent).Caption := TranslateString(TCaptionAccess(CurComponent).Caption);

    if CurComponent is TAction then
    begin
      // caption
      TAction(CurComponent).Caption := TranslateString(TAction(CurComponent).Caption);
      // hint
      TAction(CurComponent).Hint := TranslateString(TAction(CurComponent).Hint);
    end;

    if CurComponent is TLabelededit then
      TLabelededit(CurComponent).EditLabel.Caption := TranslateString(TLabelededit(CurComponent).EditLabel.Caption);

    if CurComponent is Timage then
    begin
      Timage(CurComponent).Hint := TranslateString(Timage(CurComponent).Hint);
    end;

    if Curcomponent is TComboboxEx then
      for i:=0 to TComboboxEx(CurComponent).itemsex.Count-1 do
      begin
      TComboboxEx(CurComponent).itemsex[i].caption:=translatestring(TComboboxEx(CurComponent).itemsex[i].caption);
      end;

    try
    if Curcomponent is TListview then
      for i:=0 to TListview(CurComponent).Columns.Count-1 do
      begin
      TListview(CurComponent).Columns[i].caption:=translatestring(TListview(CurComponent).Columns[i].caption);
      end;
    except
    end;

    if CurComponent is TMenuItem then
    begin
      TMenuItem(CurComponent).Caption := TranslateString(TMenuItem(CurComponent).Caption);
      TMenuItem(CurComponent).Hint := TranslateString(TMenuItem(CurComponent).Hint);
    end;

  end;
end;

{-----------------------------------------------------------------------------
  Procedure: TranslateActionToolBar
  Purpose  : Translates a ActionBarItem and all of its childs.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: const ActionBarItem: TActionBarItem
  Result   : None
  History  :
    09.12.2003 10:53:28
-----------------------------------------------------------------------------}
procedure TranslateActionBarItem(const ActionBarItem: TActionBarItem);
var
  ActionItem: integer;
begin
  if (ActionBarItem.ContextItems.Count > 0) then
  begin
    for ActionItem := 0 to ActionBarItem.ContextItems.Count-1 do
    begin
      TranslateActionItem(ActionBarItem.ContextItems[ActionItem]);
    end;
  end;
  if (ActionBarItem.HasItems) then
  begin
    for ActionItem := 0 to ActionBarItem.Items.Count-1 do
    begin
      TranslateActionItem(ActionBarItem.Items[ActionItem]);
    end;
  end;
end;

{-----------------------------------------------------------------------------
  Procedure: TranslateActionItem
  Purpose  : Translates the ActionItem and all child items.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: const ActionItem: TActionItem
  Result   : None
  History  :
    09.12.2003 10:43:17
-----------------------------------------------------------------------------}
procedure TranslateActionItem(const ActionItem: TActionClientItem);
var
  SubItems: integer;
begin
  ActionItem.Caption := TranslateString(ActionItem.Caption);
  if (ActionItem.ContextItems.Count > 0) then
  begin
    for SubItems := 0 to ActionItem.ContextItems.Count-1 do
    begin
      TranslateActionItem(ActionItem.ContextItems[SubItems]);
    end;
  end;
  if ActionItem.HasItems then
  begin
    for SubItems := 0 to ActionItem.Items.Count-1 do
    begin
      TranslateActionItem(ActionItem.Items[SubItems]);
    end;
  end;  
end;

end.
