(*
 *   FeedReader
 *   Copyright (C) 2003 The FeedReader Developement Group
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *)
unit Misc;

interface

Uses
  Windows, ComCtrls, Forms,Graphics,
  XDOM_2_3,
  HttpProt, IdHTTP, IdCoderMIME,commctrl,contnrs;
type ext_info=class
  name,desc,author,version:string;
  ext_version:integer;
  action_open_item,action_update,action_config:boolean;
  action_open_item_name,action_update_type:string;
end;
type Talertboxdata = class
title,body:string;
node:ttreenode;
showed:boolean;
end;
type
  THeadline = class
    tiitel, link, linkdoc, sisu, image, topic, creator: string;
    date, comments, section, other,modified: string;
    SourceDescription, SourceURL: String;
    deleted: boolean;
    read,flag,rating: integer;
    hlid, owner: real;
    enclosure: boolean;
    enc_url, enc_size, enc_type: string;
    guid: String;
    itemtype:string;
  end;
  Tfeedinfo = class
    link, htmlurl, un, pw, feedtype,LastModified, description, image, tiitel, sisu: string;
    updateperiod, id, read, unreadcount: integer;
    feedid: real;
    Hint: String;
    ArchiveSize,feediconindex: Integer;
    AlwaysShowLink,allowfromweb: Boolean;

    UpdateThread: TObject;
  end;
  type tgetinfo=function:ext_info;
  type taction_configure=procedure;
  type taction_update=function(URL, LastModified, Username, Password:string):tobjectlist;
  type taction_open_item=procedure(headline:theadline);
  type tshow_action_open_item=function(headline:theadline):ShortString;
  type tgetrating=function(headline:theadline):integer;
  THeadlinearray = record
    tiitel, link, linkdoc, sisu, image, topic, creator: string;
    date, comments, section, other: string;
    SourceDescription, SourceURL: String;
    deleted: boolean;
    IsRead,read: integer;
    hlid, owner: real;
    enclosure: boolean;
    enc_url, enc_size, enc_type: string;
    guid: String;
    itemtype:string;
  end;

  TSubscribedFeed = record
    unreadcount, feedtype, LastModified, username, password, parent, link,
    title, description, image, htmlurl,
    unread: string;
    updateperiod: Integer;
    feedid: real;
    ArchiveSize: Integer;
    AlwaysShowLink,allowfromweb: Boolean;

    LastUpdateTime: TDateTime;
    UpdateThread: TObject;
  end;

  TFeedProvider = record
    docid, utime: integer;
    name, link: string;
  end;

  TFeedProviderListItem = record
    deleted, username, password, owner: integer;
    link, title: string;
  end;

  TFeedProviderItem = class
    link, feedtype, name: string;
    docid, utime: integer;
  end;



Var
  gSubscribedFeeds: array of TSubscribedFeed;
  gHeadlines: TObjectList;
  gAlertBoxData: Tobjectlist;
  gFeedProviderList: array of TFeedProvider;
  gFeedProviderListItems: array of TFeedProviderListItem;
  dllhandle : THandle;
  gCritSect: TRTLCriticalSection;

procedure DrawGradient(ACanvas: TCanvas; Rect: TRect;Horicontal: Boolean; Colors: array of TColor);

Procedure place_searchbox;
Procedure CheckSingleInstance( ASplashScreen: TForm );
procedure update_external_feedsource(node:ttreenode);
Procedure AddChildToDomDoc( DomDocument: TdomDocument; ParentNode: TDomNode; Const ElementName, TextContent: String; AddIfEmpty: Boolean  = True );
Procedure AddCDataChildToDomDoc( DomDocument: TdomDocument; ParentNode: TDomNode; Const ElementName, TextContent: String; AddIfEmpty: Boolean  = True );
Procedure SaveDomDocumentToFile( DomDocument: TDomDocument; Const Filename: String );

procedure LoadHeadlines( OwnerForm: TForm );
procedure LoadFilters( OwnerForm: TForm );

procedure LoadFeedProviderItems( OwnerForm: TForm );
procedure LoadSubscribedFeeds( OwnerForm: TForm );
procedure LoadFeedProviderList( OwnerForm: TForm );
procedure SaveHeadlines( OwnerForm: TForm );
procedure SaveFilters( OwnerForm: TForm );

procedure SaveFeedProviderList( OwnerForm: TForm; RootNode: TTreeNode );
procedure SaveFeedProviderItems( OwnerForm: TForm );
Procedure SaveSubscribedFeeds( OwnerForm: TForm; RootNode: TTreeNode );

Function LoadFileIntoXDom( Const FileName: String; XmlToDomParser: TXmlToDomParser ): TdomDocument;

Function GetHeadlineIndex( ownlynotdeleted: boolean; Owner: real; Const headline, link: string; read: Integer): Integer;overload;
Function GetHeadlineIndex( ownlynotdeleted: boolean;guid:string): Integer;overload;
Function GetHeadlineIndex( ownlynotdeleted: boolean;itemid:real): Integer;overload;

procedure SortHeadlines(var List: tobjectlist);

Procedure InitHttpClientProxySettings( Client: TIdHTTP ); overload;
Procedure InitHttpClientProxySettings( Client: THttpCli ); overload;

Procedure InvalidateTreeNode( TreeView: TCustomTreeView; Node: TTreeNode );
function return_extension_open_item_actions(headline:theadline):string;
Function NodeIsChildOf( Node, Parent: TTreeNode ): Boolean;

function Sto_GetFileVersion(const FileName: String; BlockCount: Word = 4): String;

function SearchFeedLink(const FeedLink: string) : boolean;
function SearchFeedID(const FeedLink: string) : real;
procedure FocusTreeViewNodeFrom(const TreeView: TTreeView; const FeedLink: string);

implementation

Uses
  Classes, Dialogs, languagesupport,SysUtils,
  FiltersForm_FRM,Formatted_XDOM,
  XFunctions,mainform_frm,  FReaderProperties, ComObj, IdCoder;

Function EncryptPassword( Const Password, Key: String ): String;
Begin
  Result := EncryptDecryptWithKey( Password, Key );
  Result := TIdEncoderMIME.EncodeString( Result );
End;

Function DecryptPassword( Const Password, Key: String ): String;
Begin
  Result := TIdDecoderMIME.DecodeString( Password );
  Result := EncryptDecryptWithKey( Result, Key );
End;

Procedure SaveDomDocumentToFile( DomDocument: TDomDocument; Const Filename: String );
Var
  OutputStream: TFileStream;
  DomToXmlParser: TPrettyDomToXmlParser;
Begin
  OutputStream:= TFileStream.create( Filename ,fmCreate );
  try
    DomToXmlParser := TPrettyDomToXmlParser.create( Nil );
    try
      DomToXmlParser.DOMImpl := DomDocument.domImplementation;
      DomToXmlParser.writeToStream( DomDocument, DomDocument.Encoding, OutputStream );
    finally
      DomToXmlParser.free;
    end;
  finally
    OutputStream.free;
  end;
End;

Procedure AddChildToDomDoc( DomDocument: TdomDocument; ParentNode: TDomNode; Const ElementName, TextContent: String; AddIfEmpty: Boolean  = True );
Var
  NewNode: TDomNode;
Begin
  If Not AddIfEmpty Then
    If Length(TextContent) = 0 Then
      Exit;

  NewNode := DomDocument.createElement( ElementName );
  NewNode.appendChild( DomDocument.createTextNode( TextContent ) );
  ParentNode.appendChild( NewNode );
End;


Procedure AddCDataChildToDomDoc( DomDocument: TdomDocument; ParentNode: TDomNode; Const ElementName, TextContent: String; AddIfEmpty: Boolean  = True );
Var
  NewNode: TDomNode;
Begin
  If Not AddIfEmpty Then
    If Length(TextContent) = 0 Then
      Exit;

  NewNode := DomDocument.createElement( ElementName );
  NewNode.appendChild( DomDocument.createCDATASection( TextContent ) );
  ParentNode.appendChild( NewNode );
End;


procedure SortHeadlines(var List: Tobjectlist);
var
  last_swap, i, j: integer;
    tmp: THeadline;
function byowner(Item1, Item2: Pointer): Integer;
begin
  if theadline(item1).owner=theadline(item2).owner then result:=0;
  if theadline(item1).owner>theadline(item2).owner then result:=1;
  if theadline(item1).owner<theadline(item2).owner then result:=-1;

end;
begin
list.Sort(@byowner);
exit;
End;


Procedure CheckSingleInstance( ASplashScreen: TForm );
Var
  fMutex: THandle;
Begin
  { create our mutex }
  fMutex := CreateMutex(nil, FALSE, @Application.Title[1]);
  try
    { test mutex to see if we need to terminate or not }
    if (GetLastError() = ERROR_ALREADY_EXISTS) or
      (WaitForSingleObject(fMutex, 100) = WAIT_TIMEOUT) or
      (fMutex = 0) then
    begin

      { keep mainform from flickering }
      Application.ShowMainForm := FALSE;

      If Assigned(ASplashScreen) Then
        ASplashScreen.Hide;

      ShowMessage('You can only have one instance of ' +
        Application.Title +
        ' running at one time.');

      { terminate our application }
      Application.Terminate;
    end;

  finally
    { free our created mutex }
    ReleaseMutex(fMutex);
  end;
End;


Procedure InitHttpClientProxySettings( Client: TIdHTTP );
Begin
  If Assigned( Client ) Then
    if gProperties.ProxyEnabled then
    begin
      Client.ProxyParams.BasicAuthentication := True;
      Client.ProxyParams.ProxyServer := gProperties.ProxyHost;
      Client.ProxyParams.ProxyPort := StrToIntDef(gProperties.ProxyPort, 8080);
      Client.ProxyParams.ProxyUsername := gProperties.ProxyUsername;
      Client.ProxyParams.ProxyPassword := gProperties.ProxyPassword;
    end;
End;


Procedure InitHttpClientProxySettings( Client: THttpCli );
Begin
  If Assigned( Client ) Then
    if gProperties.ProxyEnabled then
    begin
      Client.Proxy := gProperties.ProxyHost;
      Client.proxyport := gProperties.ProxyPort;
      Client.Proxyusername := gProperties.ProxyUsername;
      Client.Proxypassword := gProperties.ProxyPassword;
    end;
End;


Function LoadFileIntoXDom( Const FileName: String; XmlToDomParser: TXmlToDomParser ): TdomDocument;

  Function ChangeInputSource( Const NewEncoding: String ): TdomDocument;
  Var
    FS: TFileStream;
    Stream: TStringStream;
    S: String;
    i, j: Integer;
  Begin
    FS := TFileStream.Create( FileName, fmOpenRead );
    Try
      Stream := TStringStream.Create( '' );
      Try
        Stream.CopyFrom( FS, FS.Size );
        S := Stream.DataString;
        FreeAndNil( Stream );

        If Length(NewEncoding) > 0 Then
          S := SysUtils.StringReplace( S, '"UTF-8"?>', '"' + NewEncoding + '"?>', [rfIgnoreCase] );

        For i := 1 To 31 Do
          For j := 1 To Length(S) Do
            If S[j] = Chr(i) Then S[j] := #32;

        Stream := TStringStream.Create( S );

        Result := XmlToDomParser.streamToDom( Stream );
      Finally
        Stream.Free;
      End;
    Finally
      FS.Free;
    End;
  End;

Begin
  Result := Nil;

  If Result = Nil Then
  Try
    Result := XmlToDomParser.fileToDom( FileName );
  Except

    Try
      Result := ChangeInputSource( '' );
    Except
      Try
        Result := ChangeInputSource( 'ISO-8859-1' );
      Except
        Result := ChangeInputSource( 'windows-1250' );
      End;
    End;

  End;
End;


procedure LoadHeadlines( OwnerForm: TForm );
var
  FileName: String;
  domdoc: tdomdocument;
  DomImplementation: tdomimplementation;
  XmlToDomParser: txmltodomparser;
  tase1, tase2, tase2a: TdomNode;
  fAdvance: Boolean;
  myheadline:theadline;
begin


  Try
    FileName := gProperties.DataDirectory + '\headlines.xml';

    if not SysUtils.FileExists( FileName ) then
      exit;

    EnterCriticalSection(gCritSect);
    try

      DomImplementation := Nil;
      XmlToDomParser := Nil;
      Try
         DomImplementation := TDomImplementation.Create(nil);
         XmlToDomParser := TXmlToDomParser.Create(nil);
         Xmltodomparser.DOMImpl := DomImplementation;

         domdoc := LoadFileIntoXDom( FileName, XmlToDomParser );

         tase1 := domdoc.documentElement.firstChild;

         while tase1<>nil do
         begin
          fAdvance := True;
          tase2 := tase1.firstChild;
          while tase2<>nil do
          begin
            if tase2.nodeType = ntElement_Node then
            begin
              if fAdvance Then Begin
                fAdvance := False;
                myheadline:=theadline.create;
                myheadline.deleted := false;
                gheadlines.add(myheadline);
              End;

              tase2.resolveEntityReferences( erReplace );
                   if tase2.NodeName = 'owner'   then theadline(gheadlines[gheadlines.count-1]).owner := strtofloatDef(tase2.TextContent, 0)
              else if tase2.NodeName = 'guid'    then theadline(gheadlines[gheadlines.count-1]).guid := tase2.TextContent
              else if tase2.NodeName = 'hlid'    then theadline(gheadlines[gheadlines.count-1]).hlid := strtofloatDef(tase2.TextContent, 0)
              else if tase2.NodeName = 'link'    then theadline(gheadlines[gheadlines.count-1]).link := tase2.TextContent
              else if tase2.NodeName = 'topic'   then theadline(gheadlines[gheadlines.count-1]).topic := tase2.TextContent
              else if tase2.NodeName = 'creator' then theadline(gheadlines[gheadlines.count-1]).creator := tase2.TextContent
              else if tase2.NodeName = 'date'    then theadline(gheadlines[gheadlines.count-1]).date := tase2.TextContent
              else if tase2.NodeName = 'modified'    then theadline(gheadlines[gheadlines.count-1]).modified := tase2.TextContent
              else if tase2.NodeName = 'section' then theadline(gheadlines[gheadlines.count-1]).section := tase2.TextContent
              else if tase2.NodeName = 'other'   then theadline(gheadlines[gheadlines.count-1]).other := tase2.TextContent
              else if tase2.NodeName = 'comments'   then theadline(gheadlines[gheadlines.count-1]).comments := tase2.TextContent
              else if tase2.NodeName = 'image'   then theadline(gheadlines[gheadlines.count-1]).image := tase2.TextContent
              else if tase2.NodeName = 'read'    then theadline(gheadlines[gheadlines.count-1]).Read := StrToIntDef(tase2.TextContent, 0)
              else if tase2.NodeName = 'rating'    then theadline(gheadlines[gheadlines.count-1]).rating := StrToIntDef(tase2.TextContent, 0)
              else if tase2.NodeName = 'flag'    then theadline(gheadlines[gheadlines.count-1]).flag := StrToIntDef(tase2.TextContent, 0)
              else if tase2.NodeName = 'src'     then theadline(gheadlines[gheadlines.count-1]).SourceDescription := tase2.TextContent
              else if tase2.NodeName = 'srcurl'  then theadline(gheadlines[gheadlines.count-1]).SourceURL := tase2.TextContent
              else if tase2.NodeName = 'enc_url'  then theadline(gheadlines[gheadlines.count-1]).enc_url := tase2.TextContent
              else if tase2.NodeName = 'enc_size'  then theadline(gheadlines[gheadlines.count-1]).enc_size := tase2.TextContent
              else if tase2.NodeName = 'enc_type'  then theadline(gheadlines[gheadlines.count-1]).enc_type := tase2.TextContent
              else if tase2.NodeName = 'enclosure'  then theadline(gheadlines[gheadlines.count-1]).enclosure := (tase2.TextContent='true')

              else if tase2.NodeName = 'title'   then begin
                                                        tase2a := tase2.firstChild;
                                                        if (tase2a <> nil) and (tase2a.nodeType = ntCDATA_Section_Node) Then
                                                          theadline(gheadlines[gheadlines.count-1]).tiitel := stringreplace(tase2a.textContent, '<_specialsign_>', '&', [rfReplaceAll])
                                                         Else Begin
                                                           // compatibility for older files
                                                           tase2a := tase2a.nextSibling;
                                                           if (tase2a <> nil) and (tase2a.nodeType = ntText_Node) Then
                                                              tase2a := tase2.findNextSiblingElement;
                                                           if (tase2a <> nil) and (tase2a.nodeType = ntCDATA_Section_Node) Then
                                                             theadline(gheadlines[gheadlines.count-1]).tiitel := stringreplace(tase2a.textContent, '<_specialsign_>', '&', [rfReplaceAll]);
                                                         End;
                                                      end
              else if tase2.NodeName = 'body'    then begin
                                                        tase2a := tase2.firstChild;
                                                        if (tase2a <> nil) and (tase2a.nodeType = ntCDATA_Section_Node) Then
                                                          theadline(gheadlines[gheadlines.count-1]).sisu := tase2a.textContent
                                                         Else Begin
                                                           // compatibility for older files
                                                           tase2a := tase2a.nextSibling;
                                                           if (tase2a <> nil) and (tase2a.nodeType = ntText_Node) Then
                                                              tase2a := tase2.findNextSiblingElement;
                                                           if (tase2a <> nil) and (tase2a.nodeType = ntCDATA_Section_Node) Then
                                                             theadline(gheadlines[gheadlines.count-1]).sisu := stringreplace(tase2a.textContent, '<_specialsign_>', '&', [rfReplaceAll]);
                                                         End;
                                                      end;

            end;

            tase2 := tase2.nextsibling;
          end;
          tase1 := tase1.NextSibling;
        end;
      Finally
        XmlToDomParser.Free;
        DomImplementation.Free;
      End;
    finally
      LeaveCriticalSection(gCritSect);
    end;
  Except
    On E: Exception Do Begin
      gHeadlines.clear;
      Windows.MessageBox( OwnerForm.Handle, PChar('Error loading headlines.'#13#10#13#10 + E.Message), 'Error', MB_ICONHAND OR MB_OK );
    End;
  End;
end;

procedure LoadFeedProviderItems( OwnerForm: TForm );
var
  FileName: String;
  domdoc: tdomdocument;
  DomImplementation: tdomimplementation;
  XmlToDomParser: txmltodomparser;
  tase1, tase2: TdomNode;
  fAdvance: Boolean;
begin
  EnterCriticalSection(gCritSect);
  try
    setlength(gFeedProviderListItems, 0);

    Try
      FileName := gProperties.DataDirectory + '\feeds.xml';

      if not fileexists( FileName ) then
        exit;

      DomImplementation := Nil;
      XmlToDomParser := Nil;
      Try
         DomImplementation := TDomImplementation.Create(nil);
         XmlToDomParser := TXmlToDomParser.Create(nil);
         Xmltodomparser.DOMImpl := DomImplementation;

         domdoc := LoadFileIntoXDom( FileName, XmlToDomParser );

         tase1 := domdoc.documentElement.firstChild;

         setlength(gFeedProviderListItems, 0);
         while tase1<>nil do
         begin

           fAdvance := True;
           tase2 := tase1.firstChild;
           while tase2<>nil do
           begin

             if tase2.nodeType = ntElement_Node then
             begin
               if fAdvance then begin
                 fAdvance := False;
                 setlength(gFeedProviderListItems, length(gFeedProviderListItems) + 1);
               end;

               tase2.resolveEntityReferences( erReplace );
                    if tase2.nodeName = 'owner' then gFeedProviderListItems[length(gFeedProviderListItems) - 1].owner := strtoint(tase2.textContent)
               else if tase2.nodeName = 'link'  then gFeedProviderListItems[length(gFeedProviderListItems) - 1].link := tase2.textContent
               else if tase2.nodeName = 'title' then gFeedProviderListItems[length(gFeedProviderListItems) - 1].title := tase2.textContent;
             end;

            tase2 := tase2.nextSibling;
          end;
          tase1 := tase1.nextSibling;
        end; // while

      Finally
        XmlToDomParser.Free;
        DomImplementation.Free;
      End;
    Except
      On E: Exception Do Begin
        setlength(gFeedProviderListItems, 0);
        Windows.MessageBox( OwnerForm.Handle, PChar('Error loading feed-provider.'#13#10#13#10 + E.Message), 'Error', MB_ICONHAND OR MB_OK );
      End;
    End;
  finally
    LeaveCriticalSection(gCritSect);
  end;

end;

procedure LoadSubscribedFeeds( OwnerForm: TForm );
var
  FileName: String;
  domdoc: tdomdocument;
  DomImplementation: tdomimplementation;
  XmlToDomParser: txmltodomparser;
  tase1, tase2: TdomNode;
  fAdvance: Boolean;
begin
  EnterCriticalSection(gCritSect);
  setlength(gSubscribedFeeds, 0);
  try
    Try
      if fileexists(gProperties.ApplicationDirectory + '\import.xml') and not fileexists(gProperties.DataDirectory + '\subscriptions.xml') then
      FileName := gProperties.ApplicationDirectory + '\import.xml'
      else
      FileName := gProperties.DataDirectory + '\subscriptions.xml';

      if not fileexists( FileName ) then
        exit;

      DomImplementation := Nil;
      XmlToDomParser := Nil;
      Try
          DomImplementation := TDomImplementation.Create(nil);
          XmlToDomParser := TXmlToDomParser.Create(nil);
          Xmltodomparser.DOMImpl := DomImplementation;

          domdoc := LoadFileIntoXDom( FileName, XmlToDomParser );

          tase1 := domdoc.documentElement.firstChild;
          while tase1 <> nil do
          begin

            fAdvance := True;
            tase2 := tase1.firstChild;
            while tase2 <> nil do
            begin
              if tase2.NodeType = ntElement_Node then Begin

                 If fAdvance Then Begin
                   fAdvance := False;
                   setlength(gSubscribedFeeds, length(gSubscribedFeeds) + 1);
                   gSubscribedFeeds[length(gSubscribedFeeds) - 1].LastUpdateTime := 0;
                   gSubscribedFeeds[length(gSubscribedFeeds) - 1].ArchiveSize := 0;
                   gSubscribedFeeds[length(gSubscribedFeeds) - 1].AlwaysShowLink := False;
                 End;

                 if tase2.NodeName = 'password' then
                 begin
                    gSubscribedFeeds[length(gSubscribedFeeds) - 1].password := DecryptPassword(tase2.textContent, 'polevajateada');
                 end;
              End;

              if tase2.nodeType = ntElement_Node then
              begin
                tase2.resolveEntityReferences( erReplace );
                     if tase2.nodeName = 'htmlurl'        then gSubscribedFeeds[length(gSubscribedFeeds) - 1].htmlurl := (tase2.textContent)
                else if tase2.nodeName = 'description'    then gSubscribedFeeds[length(gSubscribedFeeds) - 1].description := tase2.textContent
                else if tase2.nodeName = 'image'          then gSubscribedFeeds[length(gSubscribedFeeds) - 1].image := tase2.textContent
                else if tase2.nodeName = 'link'           then gSubscribedFeeds[length(gSubscribedFeeds) - 1].link := tase2.textContent
                else if tase2.nodeName = 'feedid'         then gSubscribedFeeds[length(gSubscribedFeeds) - 1].feedid := strtofloat(tase2.textContent)

                else if tase2.nodeName = 'title'          then gSubscribedFeeds[length(gSubscribedFeeds) - 1].title := tase2.textContent
                else if tase2.nodeName = 'read'           then gSubscribedFeeds[length(gSubscribedFeeds) - 1].unread := tase2.textContent
                else if tase2.nodeName = 'unreadcount'    then gSubscribedFeeds[length(gSubscribedFeeds) - 1].unreadcount := tase2.textContent
                else if tase2.nodeName = 'feedtype'       then gSubscribedFeeds[length(gSubscribedFeeds) - 1].feedtype := tase2.textContent
                else if tase2.nodeName = 'parent'         then gSubscribedFeeds[length(gSubscribedFeeds) - 1].parent := tase2.textContent
                else if tase2.nodeName = 'username'       then gSubscribedFeeds[length(gSubscribedFeeds) - 1].username := tase2.textContent
                else if tase2.nodeName = 'updateperiod'   then gSubscribedFeeds[length(gSubscribedFeeds) - 1].updateperiod := StrToIntDef(tase2.textContent, 3)
                else if tase2.nodeName = 'archivesize'    then gSubscribedFeeds[length(gSubscribedFeeds) - 1].ArchiveSize := StrToIntDef( tase2.textContent, 0)
                else if tase2.nodeName = 'LastModified'    then gSubscribedFeeds[length(gSubscribedFeeds) - 1].LastModified := tase2.textContent
                else if tase2.nodeName = 'alwaysshowlink' then gSubscribedFeeds[length(gSubscribedFeeds) - 1].AlwaysShowLink := ( 0 <> StrToIntDef( tase2.textContent, 0) )
                else if tase2.nodeName = 'allowaccessfromweb' then gSubscribedFeeds[length(gSubscribedFeeds) - 1].allowfromweb := ( 0 <> StrToIntDef( tase2.textContent, 0) );
              end;

              tase2 := tase2.nextSibling;
            end;

            If Length(gSubscribedFeeds) > 0 Then Begin
              If (gSubscribedFeeds[length(gSubscribedFeeds) - 1].ArchiveSize > 9) Then
                If (gSubscribedFeeds[length(gSubscribedFeeds) - 1].ArchiveSize <> UnlimitedArchiveSizeIdValue) And (gSubscribedFeeds[length(gSubscribedFeeds) - 1].ArchiveSize <> DefaultArchiveSizeIdValue) Then
                  gSubscribedFeeds[length(gSubscribedFeeds) - 1].ArchiveSize := 9;
              If (gSubscribedFeeds[length(gSubscribedFeeds) - 1].ArchiveSize < 0) Then
                gSubscribedFeeds[length(gSubscribedFeeds) - 1].ArchiveSize := 0;
            End;


            tase1 := tase1.nextSibling;
          end;
      Finally
        XmlToDomParser.Free;
        DomImplementation.Free;
      End;
    Except
      On E: Exception Do Begin
        Renamefile(gProperties.DataDirectory + '\subscriptions.xml',gProperties.DataDirectory + '\subscriptions_bad.xml');
        setlength(gSubscribedFeeds, 0);
        Windows.MessageBox( OwnerForm.Handle, PChar('Error loading subscribed feeds.'#13#10#13#10 + E.message), 'Error', MB_ICONHAND OR MB_OK );
      End;
    End;
  finally
    LeaveCriticalSection(gCritSect);
  end;
end;

procedure LoadFeedProviderList( OwnerForm: TForm );
Var
  domdoc: tdomdocument;
  DomImplementation: tdomimplementation;
  XmlToDomParser: txmltodomparser;
  tase1, tase2: TdomNode;
  FileName: String;
  fAdvance: Boolean;
begin
  EnterCriticalSection(gCritSect);
  try
    setlength(gFeedProviderList, 0);

    Try
      FileName := gProperties.DataDirectory + '\feedlists.xml';

      if not fileexists( FileName ) then
        exit;

      DomImplementation := Nil;
      XmlToDomParser := Nil;
      Try
          DomImplementation := TDomImplementation.Create(nil);
          XmlToDomParser := TXmlToDomParser.Create(nil);
          Xmltodomparser.DOMImpl := DomImplementation;

          domdoc := LoadFileIntoXDom( FileName, XmlToDomParser );

          tase1 := domdoc.documentElement.firstChild;
          while tase1 <> nil do
          begin

            tase2 := tase1.firstChild;
            fAdvance := True;
            while tase2<>nil do
            begin

              if tase2.nodeType = ntElement_Node then
              begin
                If fAdvance Then Begin
                  fAdvance := False;
                  setlength(gFeedProviderList, length(gFeedProviderList) + 1);
                End;

                tase2.resolveEntityReferences( erReplace );
                     if tase2.nodeName = 'url'   then gFeedProviderList[length(gFeedProviderList) - 1].link  := tase2.textContent
                else if tase2.nodeName = 'title' then gFeedProviderList[length(gFeedProviderList) - 1].name  := tase2.textContent
                else if tase2.nodeName = 'docid' then gFeedProviderList[length(gFeedProviderList) - 1].docid := strtoint(tase2.textContent)
                else if tase2.nodeName = 'utime' then gFeedProviderList[length(gFeedProviderList) - 1].utime := strtoint(tase2.textContent);

              end;

              tase2 := tase2.nextSibling;
            end;
            tase1 := tase1.nextSibling;
          end;
      Finally
        XmlToDomParser.Free;
        DomImplementation.Free;
      End;
    Except
      On E: Exception Do Begin
        setlength(gFeedProviderList, 0);
        Windows.MessageBox( OwnerForm.Handle, PChar('Error loading feed-provider-list.'#13#10#13#10 + E.Message), 'Error', MB_ICONHAND OR MB_OK );
      End;
    End;
  finally
    LeaveCriticalSection(gCritSect);
  end;
end;

procedure SaveHeadlines( OwnerForm: TForm );
Var
  DomImplementation: TDomImplementation;
  DomDoc: tdomdocument;
  Element: TdomElement;
  cac: TDomElement;
  j1,i: Integer;
  subscriptionlist:string;
begin
  Try
    //Save all headlines
    //generate list of all subscription id-s
    subscriptionlist:=';';
    for i:=low(gsubscribedfeeds) to high(gsubscribedfeeds) do subscriptionlist:=subscriptionlist+floattostr(gsubscribedfeeds[i].feedid)+'; ';
    //end of generating list

    SortHeadlines(gHeadlines);

    DomImplementation := Nil;
    DomDoc := Nil;
    Try
      DomImplementation := TDomImplementation.Create( Nil );
      DomDoc := TdomDocument.Create( DomImplementation, Nil );
      DomDoc.encoding := 'UTF-8';

      Element := DomDoc.createElement( 'headlines' );
      DomDoc.appendChild( Element );

      EnterCriticalSection(gCritSect);
      try
        for j1 := 0 to gHeadlines.count - 1 do Begin
           if not theadline(gHeadlines[j1]).deleted and (pos(formatfloat('0',theadline(gHeadlines[j1]).owner),subscriptionlist)>0) then
           begin
              cac := DomDoc.createElement( 'item' );
              Element.appendChild(cac);

              AddChildToDomDoc( DomDoc, cac, 'hlid', floattostr(theadline(gHeadlines[j1]).hlid) );
              AddChildToDomDoc( DomDoc, cac, 'guid', theadline(gHeadlines[j1]).guid);
              AddChildToDomDoc( DomDoc, cac, 'owner', floattostr(theadline(gHeadlines[j1]).owner) );
              AddCDataChildToDomDoc( DomDoc, cac, 'title', stringreplace(theadline(gHeadlines[j1]).tiitel, '&', '<_specialsign_>', [rfReplaceAll]) );
              AddChildToDomDoc( DomDoc, cac, 'link', theadline(gHeadlines[j1]).link, false );
              AddChildToDomDoc( DomDoc, cac, 'src', theadline(gHeadlines[j1]).SourceDescription, false );
              AddChildToDomDoc( DomDoc, cac, 'srcurl', theadline(gHeadlines[j1]).SourceURL, false );
              AddCDataChildToDomDoc( DomDoc, cac, 'body', theadline(gHeadlines[j1]).sisu, false );
              AddChildToDomDoc( DomDoc, cac, 'topic', theadline(gHeadlines[j1]).topic, false );
              AddChildToDomDoc( DomDoc, cac, 'creator', theadline(gHeadlines[j1]).creator, false );
              AddChildToDomDoc( DomDoc, cac, 'date', theadline(gHeadlines[j1]).date, false );
              if (trim(theadline(gHeadlines[j1]).modified)<>'') then AddChildToDomDoc( DomDoc, cac, 'modified', theadline(gHeadlines[j1]).modified, false );
              AddChildToDomDoc( DomDoc, cac, 'section', theadline(gHeadlines[j1]).section, false );
              AddChildToDomDoc( DomDoc, cac, 'image', theadline(gHeadlines[j1]).image, false );
              AddChildToDomDoc( DomDoc, cac, 'comments', theadline(gHeadlines[j1]).comments, false );
              AddChildToDomDoc( DomDoc, cac, 'other', theadline(gHeadlines[j1]).other, false );
              AddChildToDomDoc( DomDoc, cac, 'read', IntToStr(theadline(gHeadlines[j1]).Read) );
              AddChildToDomDoc( DomDoc, cac, 'rating', IntToStr(theadline(gHeadlines[j1]).rating) );
              AddChildToDomDoc( DomDoc, cac, 'flag', IntToStr(theadline(gHeadlines[j1]).flag) );
              if theadline(gHeadlines[j1]).enclosure then AddChildToDomDoc( DomDoc, cac, 'enclosure', 'true' );
              AddChildToDomDoc( DomDoc, cac, 'enc_url', theadline(gHeadlines[j1]).enc_url );
              AddChildToDomDoc( DomDoc, cac, 'enc_size', theadline(gHeadlines[j1]).enc_size );
              AddChildToDomDoc( DomDoc, cac, 'enc_type', theadline(gHeadlines[j1]).enc_type );

           end; //if not deleted
        end; //for j1
      finally
        LeaveCriticalSection(gCritSect);
      end;

      SaveDomDocumentToFile( DomDoc, gProperties.DataDirectory + '\headlines.xml' );

    Finally
      DomDoc.Free;
      DomImplementation.Free;
    End;
  Except
    On E: Exception Do
      Windows.MessageBox( OwnerForm.Handle, PChar('Error saving headlines.'#13#10#13#10 + E.message), 'Error', MB_ICONHAND OR MB_OK );
  End;
end;

procedure SaveFeedProviderItems( OwnerForm: TForm );
Var
  DomImplementation: TDomImplementation;
  DomDoc: tdomdocument;
  Element: TdomElement;
  cac: TDomElement;
  j1: Integer;
begin
  Try
    DomImplementation := Nil;
    DomDoc := Nil;
    Try
      DomImplementation := TDomImplementation.Create( Nil );
      DomDoc := TdomDocument.Create( DomImplementation, Nil );
      DomDoc.encoding := 'UTF-8';

      Element := DomDoc.createElement( 'afeeds' );
      DomDoc.appendChild( Element );

      EnterCriticalSection(gCritSect);
      try
        for j1 := 0 to length(gFeedProviderListItems) - 1 do
        begin
          if gFeedProviderListItems[j1].deleted = 0 then
          begin
            cac := DomDoc.createElement( 'item' );
            Element.appendChild( cac );

            AddChildToDomDoc( DomDoc, cac, 'owner', IntToStr(gFeedProviderListItems[j1].owner) );
            AddChildToDomDoc( DomDoc, cac, 'title', gFeedProviderListItems[j1].title );
            AddChildToDomDoc( DomDoc, cac, 'link', gFeedProviderListItems[j1].link );

          end; // if not deleted
        end; // for
      finally
        LeaveCriticalSection(gCritSect);
      end;

      SaveDomDocumentToFile( DomDoc, gProperties.DataDirectory + '\feeds.xml' );

    Finally
      DomDoc.Free;
      DomImplementation.Free;
    End;
  Except
    On E: Exception Do
      Windows.MessageBox( OwnerForm.Handle, PChar('Error saving feed-provider-list.'#13#10#13#10 + E.Message), 'Error', MB_ICONHAND OR MB_OK );
  End;
end;


procedure SaveFeedProviderList( OwnerForm: TForm; RootNode: TTreeNode );
Var
  DomImplementation: TDomImplementation;
  DomDoc: tdomdocument;
  Element: TdomElement;
  cac: TDomElement;
  Node: TTreeNode;
begin
  Try
    DomImplementation := Nil;
    DomDoc := Nil;
    Try
      DomImplementation := TDomImplementation.Create( Nil );
      DomDoc := TdomDocument.Create( DomImplementation, Nil );
      DomDoc.encoding := 'UTF-8';

      Element := DomDoc.createElement( 'feedlists' );
      DomDoc.appendChild( Element );

      node := RootNode.getFirstChild; //gAvailableFeedsTreeNode
      while (node<>nil) and (node.data<>nil) do
      begin

        if node.Level = 1 then
        begin

          cac := DomDoc.createElement( 'item' );
          Element.appendChild( cac );

          AddChildToDomDoc( DomDoc, cac, 'url', TFeedProviderItem(node.data).link );
          AddChildToDomDoc( DomDoc, cac, 'title', TFeedProviderItem(node.data).name );
          AddChildToDomDoc( DomDoc, cac, 'docid', IntToStr(TFeedProviderItem(node.data).docid) );
          AddChildToDomDoc( DomDoc, cac, 'utime', IntToStr(TFeedProviderItem(node.data).utime) );
          AddChildToDomDoc( DomDoc, cac, 'feedtype', TFeedProviderItem(node.data).feedtype );
          AddChildToDomDoc( DomDoc, cac, 'link', TFeedProviderItem(node.data).link );
        End;

        node := node.GetNext;
      end; // while

      SaveDomDocumentToFile( DomDoc, gProperties.DataDirectory + '\feedlists.xml' );

    Finally
      DomDoc.Free;
      DomImplementation.Free;
    End;
  Except
    On E: Exception Do
      Windows.MessageBox( OwnerForm.Handle, PChar('Error saving feed-provider-list.'#13#10#13#10 + E.Message), 'Error', MB_ICONHAND OR MB_OK );
  End;
end;


Procedure SaveSubscribedFeeds( OwnerForm: TForm; RootNode: TTreeNode );
Var
  DomImplementation: TDomImplementation;
  DomDoc: tdomdocument;
  node: TTreeNode;
  Element: TdomElement;
  cac: TDomElement;
  i: Integer;
begin
  EnterCriticalSection(gCritSect);
  try
    Try
       DomImplementation := Nil;
       DomDoc := Nil;
       Try
         DomImplementation := TDomImplementation.Create( Nil );
         DomDoc := TdomDocument.Create( DomImplementation, Nil );
         DomDoc.encoding := 'UTF-8';

         Element := DomDoc.createElement( 'feeds' );
         DomDoc.appendChild( Element );

         node := RootNode.getFirstChild; //gMyFeedsTreeNode
         while (node<>nil) and (node.data<>nil) do
         begin
           cac := DomDoc.createElement( 'item' );
           Element.appendChild( cac );

           AddChildToDomDoc( DomDoc, cac, 'feedid', FloatToStr( Tfeedinfo(node.data).feedid) );
           AddChildToDomDoc( DomDoc, cac, 'title', Tfeedinfo(node.data).tiitel );
           AddChildToDomDoc( DomDoc, cac, 'description', Tfeedinfo(node.data).description );
           AddChildToDomDoc( DomDoc, cac, 'feedtype', Tfeedinfo(node.data).feedtype );
           if node.Parent <> RootNode then AddChildToDomDoc( DomDoc, cac, 'parent', Tfeedinfo(node.parent.data).feedtype );
           if Tfeedinfo(node.Data).feedtype<>'folder' then
           begin
             AddChildToDomDoc( DomDoc, cac, 'archivesize', IntToStr(Tfeedinfo(node.data).ArchiveSize) );
             AddChildToDomDoc( DomDoc, cac, 'alwaysshowlink', IntToStr( Ord(Tfeedinfo(node.data).AlwaysShowLink) ) );
             AddChildToDomDoc( DomDoc, cac, 'allowaccessfromweb', IntToStr( Ord(Tfeedinfo(node.data).allowfromweb) ) );
             AddChildToDomDoc( DomDoc, cac, 'htmlurl', Tfeedinfo(node.data).htmlurl );
             AddChildToDomDoc( DomDoc, cac, 'image', Tfeedinfo(node.data).image );
             AddChildToDomDoc( DomDoc, cac, 'read', IntToStr(Tfeedinfo(node.data).read) );
             AddChildToDomDoc( DomDoc, cac, 'unreadcount', IntToStr(Tfeedinfo(node.data).unreadcount) );
             AddChildToDomDoc( DomDoc, cac, 'updateperiod', IntToStr(Tfeedinfo(node.data).updateperiod) );
             For i := Low(gSubscribedFeeds) To High(gSubscribedFeeds) Do
              If Tfeedinfo(node.data).feedid = gSubscribedFeeds[i].feedid Then Begin
              AddChildToDomDoc( DomDoc, cac, 'LastModified', gSubscribedfeeds[i].LastModified);
              break;
            End;

             AddChildToDomDoc( DomDoc, cac, 'link', Tfeedinfo(node.data).link );
             AddChildToDomDoc( DomDoc, cac, 'username', Tfeedinfo(node.data).un );
             AddChildToDomDoc( DomDoc, cac, 'password', EncryptPassword(Tfeedinfo(node.data).pw, 'polevajateada') );
           end;

           node := node.GetNext;
         end; // while

         SaveDomDocumentToFile( DomDoc, gProperties.DataDirectory + '\subscriptions.xml' );

       Finally
        DomDoc.Free;
        DomImplementation.Free;
      End;
    Except
      On E: Exception Do
        Windows.MessageBox( OwnerForm.Handle, PChar('Error saving subscribed-feeds.'#13#10#13#10 + E.Message), 'Error', MB_ICONHAND OR MB_OK );
    End;
  finally
    LeaveCriticalSection(gCritSect);
  end;
end;

//find article by headline ID
Function GetHeadlineIndex( ownlynotdeleted: boolean;itemid : real): Integer;overload;
var
  i:integer;
  begin
  try
    for i := 0 to gHeadlines.count - 1 do
      if ((not ownlynotdeleted) or (not theadline(theadline(gHeadlines[i])).deleted)) and (theadline(theadline(gHeadlines[i])).hlid=itemid)
        then begin
        Result := i;
        Exit;
    end;
  finally
  end;

  Result := -1;
  end;
//find article by guid
Function GetHeadlineIndex( ownlynotdeleted: boolean;Guid : string): Integer;overload;
var
  i: integer;
begin
  try
    for i := 0 to gHeadlines.count - 1 do
      if ((not ownlynotdeleted) or (not theadline(theadline(gHeadlines[i])).deleted)) and (Guid = theadline(gHeadlines[i]).guid)
        then begin
        Result := i;
        Exit;
    end;
  finally
  end;

  Result := -1;
end;
//find article by content
Function GetHeadlineIndex( ownlynotdeleted: boolean;Owner: real; Const headline, link: string; read: Integer): Integer;overload;
var
  i: integer;
begin
//  EnterCriticalSection(gCritSect);
  try
    for i := 0 to gHeadlines.count - 1 do
    begin

      if ((not ownlynotdeleted) or (not theadline(theadline(gHeadlines[i])).deleted)) and
         ((owner = theadline(gHeadlines[i]).owner) or (owner = 0)) and
         ((read = theadline(gHeadlines[i]).Read) or (read = -1)) and
         ((headline = '') or (pos(headline, theadline(gHeadlines[i]).tiitel) > 0)) and
         ((link = '') or (pos(link, theadline(gHeadlines[i]).link) > 0))
      then begin
        Result := i;
        Exit;
      end;

    end;
  finally
//    LeaveCriticalSection(gCritSect);
  end;

  Result := -1;
end;


Procedure InvalidateTreeNode( TreeView: TCustomTreeView; Node: TTreeNode );
Var
  RC: TRect;
Begin
  If (Node <> Nil) And (TreeView <> Nil) Then Begin
    RC := Node.DisplayRect( False );
    RC.Right := RC.Left + Screen.Width;
    InvalidateRect( TreeView.Handle, @RC, True );
  End;
End;


Function NodeIsChildOf( Node, Parent: TTreeNode ): Boolean;
Var
  TmpNode: TTreeNode;
Begin
  Result := False;

  If (Not Assigned(Node)) Or (Not Assigned(Parent)) Then
    Exit;

  TmpNode := Node;
  While (TmpNode <> Nil) And (TmpNode.Parent <> Parent) Do
    TmpNode := TmpNode.Parent;

  Result := (TmpNode <> Nil) And (TmpNode.Parent = Parent);
End;

////////////////////////////////////////////////////////////////
// this function reads the file ressource of "FileName" and returns
// the version number. "BlockCount" should be between 1-4 and means
// the precision of the version number, example:
//   BlockCount 3 -> "2.03.04"
function Sto_GetFileVersion(const FileName: String; BlockCount: Word = 4): String;
var
  iBufSize: DWORD;
  iRes: DWORD;
  pBuf: Pointer;
  pFileInfo: Pointer;
  iVer: Array[1..4] of Word;
begin
  // set default value
  Result := '';
  if (BlockCount < 1) or (BlockCount > 4) then
    BlockCount := 4;
  // does a version resource exist ?
  iBufSize := GetFileVersionInfoSize(PChar(FileName), iRes);
  if (iBufSize > 0) then
  begin
    GetMem(pBuf, iBufSize);
    try
    // get information
    GetFileVersionInfo(PChar(FileName), 0, iBufSize, pBuf);
    VerQueryValue(pBuf, '\', pFileInfo, iRes);
    // read version blocks
    iRes := PVSFixedFileInfo(pFileInfo)^.dwFileVersionMS;
    iVer[1] := HiWord(iRes);
    iVer[2] := LoWord(iRes);
    iRes := PVSFixedFileInfo(pFileInfo)^.dwFileVersionLS;
    iVer[3] := HiWord(iRes);
    iVer[4] := LoWord(iRes);
    finally
      FreeMem(pBuf);
    end;
    // format result string
    for iRes := 1 to BlockCount do
    begin
      if iRes = 1 then
        Result := IntToStr(iVer[iRes])
      else
        Result := Result + '.' + Format('%.2d', [iVer[iRes]]);
    end;
  end;
end;

{-----------------------------------------------------------------------------
  Procedure: SearchFeedLink
  Purpose  : Searches in the subscribed feeds for the given feed link.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: const FeedLink: string
  Result   : boolean
  History  :
    02.01.2004 22:04:22
-----------------------------------------------------------------------------}
function SearchFeedLink(const FeedLink: string) : boolean;
var
  i: integer;
  Link: string;
begin
  Link := AnsiLowercase(FeedLink);
  if (Link = '') then
  begin
    result := false;
    exit;
  end;
  i := 1;
  result := false;
  EnterCriticalSection(gCritSect);
  try
    while (i < Length(gSubscribedFeeds)) and (not result) do
    begin
      result := AnsiLowercase(gSubscribedFeeds[i].link) = Link;
      Inc(i);
    end;
  finally
    LeaveCriticalSection(gCritSect);
  end;
end;

function SearchFeedID(const FeedLink: string) : real;
var
  i: integer;
  Link: string;
begin
  Link := AnsiLowercase(FeedLink);
  if (Link = '') then
  begin
    result := -1;
    exit;
  end;
  i := 1;
  result := -1;
  EnterCriticalSection(gCritSect);
  try
    while (i < Length(gSubscribedFeeds)) and (result = -1) do
    begin
      if AnsiLowercase(gSubscribedFeeds[i].link) = Link then
        result := gSubscribedFeeds[i].feedid;
      Inc(i);
    end;
  finally
    LeaveCriticalSection(gCritSect);
  end;
end;

procedure FocusTreeViewNodeFrom(const TreeView: TTreeView; const FeedLink: string);
begin
// TreeView.Items
end;
procedure SaveFilters( OwnerForm: TForm );
//save all filters to file
Var
  DomImplementation: TDomImplementation;
  DomDoc: tdomdocument;
  node: TTreeNode;
  Element: TdomElement;
  cac: TDomElement;
  i: Integer;
begin
  EnterCriticalSection(gCritSect);
  try
    Try
       DomImplementation := Nil;
       DomDoc := Nil;
       Try
         DomImplementation := TDomImplementation.Create( Nil );
         DomDoc := TdomDocument.Create( DomImplementation, Nil );
         DomDoc.encoding := 'UTF-8';

         Element := DomDoc.createElement( 'filters' );
         DomDoc.appendChild( Element );

         for i:=0 to filtersform.listview1.items.Count-1 do
           begin
           if feednumber(filtersform.listview1.items[i].data).action>-1 then
           begin
           cac := DomDoc.createElement( 'filter2' );
           Element.appendChild( cac );

           AddChildToDomDoc( DomDoc, cac, 'filtername', filtersform.listview1.items[i].Caption );
           AddChildToDomDoc( DomDoc, cac, 'feed', filtersform.listview1.items[i].subitems[0] );
           AddChildToDomDoc( DomDoc, cac, 'feedid', floattostr(feednumber(filtersform.listview1.items[i].data).id));
           AddChildToDomDoc( DomDoc, cac, 'phrase', filtersform.listview1.items[i].subitems[1] );
           AddChildToDomDoc( DomDoc, cac, 'action', inttostr(feednumber(filtersform.listview1.items[i].data).action) );
           AddChildToDomDoc( DomDoc, cac, 'option', inttostr(feednumber(filtersform.listview1.items[i].data).option));
           end;
           end;
            // for

         SaveDomDocumentToFile( DomDoc, gProperties.DataDirectory + '\Filters.xml' );

       Finally
        DomDoc.Free;
        DomImplementation.Free;
      End;
    Except
      On E: Exception Do
        Windows.MessageBox( OwnerForm.Handle, PChar('Error saving filters.'#13#10#13#10 + E.Message), 'Error', MB_ICONHAND OR MB_OK );
    End;
  finally
    LeaveCriticalSection(gCritSect);
  end;

end;
procedure LoadFilters( OwnerForm: TForm );
var
  FileName: String;
  domdoc: tdomdocument;
  DomImplementation: tdomimplementation;
  XmlToDomParser: txmltodomparser;
  tase1, tase2, tase2a,z: TdomNode;
  item:tlistitem;
  x2: tdomnodelist;
  i:integer;
begin

  Try
    FileName := gProperties.DataDirectory + '\filters.xml';

    if not SysUtils.FileExists( FileName ) then
      exit;

    EnterCriticalSection(gCritSect);
    try

      DomImplementation := Nil;
      XmlToDomParser := Nil;
      Try
         DomImplementation := TDomImplementation.Create(nil);
         XmlToDomParser := TXmlToDomParser.Create(nil);
         Xmltodomparser.DOMImpl := DomImplementation;

         domdoc := LoadFileIntoXDom( FileName, XmlToDomParser );

         tase1 := domdoc.documentElement.firstChild;
         x2:=domdoc.getElementsByTagName('filter2');
         for i:=0 to x2.length-1 do
            begin
            item:=filtersform.listview1.items.add;
            item.SubItems.Add('');
            item.SubItems.Add('');
            item.SubItems.Add('');
            item.SubItems.Add('');
            item.data:=feednumber.create;
            feednumber(item.data).id:=0;
            z := x2.item(i);
            if z.getFirstChildElement('phrase')<>nil then
              begin
              item.subitems[1]:=z.getFirstChildElement('phrase').TextContent;
              feednumber(item.data).keyword:=z.getFirstChildElement('phrase').TextContent;

            if z.getFirstChildElement('action')<>nil then
            begin
              try
              feednumber(item.data).action:=-1;
              feednumber(item.data).action:=strtoint(z.getFirstChildElement('action').TextContent);
              item.subitems[2]:=translatestring(filtersform.combobox1.items[feednumber(item.data).action]);
              except
              end;
            end;

              end;

            if z.getFirstChildElement('option')<>nil then
            begin
              try
              feednumber(item.data).option:=-1;
              feednumber(item.data).option:=strtoint(z.getFirstChildElement('option').TextContent);
              if feednumber(item.data).action=0 then
                item.subitems[3]:=translatestring(filtersform.colorbox1.ColorNames[feednumber(item.data).option])
              else if feednumber(item.data).action=2 then
                item.subitems[3]:=translatestring(filtersform.combo_flagcolors.Items[feednumber(item.data).option])
              else if feednumber(item.data).action=3 then
                item.subitems[3]:=filtersform.combo_folders.Items[feednumber(item.data).option];

              except
              end;

              end;


            if z.getFirstChildElement('feedid')<>nil then feednumber(item.data).id:=strtofloat(z.getFirstChildElement('feedid').TextContent);
            if z.getFirstChildElement('feed')<>nil then item.subitems[0]:=z.getFirstChildElement('feed').TextContent;
            if z.getFirstChildElement('filtername')<>nil then item.caption:=z.getFirstChildElement('filtername').TextContent;
            end;
      Finally
        XmlToDomParser.Free;
        DomImplementation.Free;
      End;
    finally
      LeaveCriticalSection(gCritSect);
    end;
  Except
    On E: Exception Do Begin
      Windows.MessageBox( OwnerForm.Handle, PChar('Error loading filters.'#13#10#13#10 + E.Message), 'Error', MB_ICONHAND OR MB_OK );
    End;
  End;
end;
function return_extension_open_item_actions(headline:theadline):string;
var
sr: TSearchRec;
  FileAttrs: Integer;
getinfo:tgetinfo;
showactionitem:tshow_action_open_item;
di:ext_info;
show:ShortString;
tmpstring:string;
begin
tmpstring:='';
fileattrs:=0;
FileAttrs := FileAttrs + faAnyFile;
if FindFirst('*.dll', FileAttrs, sr) = 0 then

    begin
      repeat
        DLLHandle := LoadLibrary(pchar(sr.name));
        if DLLHandle <> 0 then
        begin
          @showactionitem := GetProcAddress(DLLHandle, 'show_action_open_item');
          if assigned(showactionitem) then
          begin
          show:='';
          try
          show:=showactionitem(headline);
          except
          end;
          if (show<>'') then tmpstring:=tmpstring+'</b> | <a href="feedreader://dllaction/'+sr.name+'"><b>'+show+'</a>';
        end;
        end;
      until FindNext(sr) <> 0;
      FindClose(sr);
    end;




return_extension_open_item_actions:=tmpstring;
end;
procedure update_external_feedsource(node:ttreenode);
Begin
end;
procedure place_searchbox;
var  PanelRect: TRect;
begin
  with mainwindow do
    begin
    statusbar1.Panels[0].Width:=statusbar1.ClientWidth-155;

    SendMessage(StatusBar1.Handle, SB_GETRECT, 3, Integer(@PanelRect));
    with PanelRect do ProgressBar1.SetBounds(Left, Top, Right - Left, Bottom - Top);

        SendMessage(StatusBar1.Handle, SB_GETRECT, 2, Integer(@PanelRect));
    with PanelRect do image_webaccess.SetBounds(Left, Top, Right - Left, Bottom - Top);

        SendMessage(StatusBar1.Handle, SB_GETRECT, 1, Integer(@PanelRect));
    with PanelRect do image_updateengine.SetBounds(Left, Top, Right - Left, Bottom - Top);
    with PanelRect do image_updateengine_notactive.SetBounds(Left, Top, Right - Left, Bottom - Top);

    image_updateengine.Hint:=languagesupport.translatestring(image_updateengine.Hint);
    image_updateengine_notactive.Hint:=translatestring(image_updateengine_notactive.Hint);
    image_webaccess.Hint:=translatestring(image_webaccess.Hint);

    searchtoolbar.SetBounds(actiontoolbar1.ClientWidth-searchtoolbar.Width,1,searchtoolbar.Width,actiontoolbar1.ClientHeight);
    edtsearchtext.top:=(searchtoolbar.height div 2)-(edtsearchtext.height div 2);
    speedbutton1.Top:=edtsearchtext.top+1;
    end;
end;
procedure DrawGradient(ACanvas: TCanvas; Rect: TRect;
  Horicontal: Boolean; Colors: array of TColor);
type
  RGBArray = array[0..2] of Byte;
var
  x, y, z, stelle, mx, bis, faColorsh, mass: Integer;
  Faktor: double;
  A: RGBArray;
  B: array of RGBArray;
  merkw: integer;
  merks: TPenStyle;
  merkp: TColor;
begin
  mx := High(Colors);
  if mx > 0 then
  begin
    if Horicontal then
      mass := Rect.Right - Rect.Left
    else
      mass := Rect.Bottom - Rect.Top;
    SetLength(b, mx + 1);
    for x := 0 to mx do
    begin
      Colors[x] := ColorToRGB(Colors[x]);
      b[x][0] := GetRValue(Colors[x]);
      b[x][1] := GetGValue(Colors[x]);
      b[x][2] := GetBValue(Colors[x]);
    end;
    merkw := ACanvas.Pen.Width;
    merks := ACanvas.Pen.Style;
    merkp := ACanvas.Pen.Color;
    ACanvas.Pen.Width := 1;
    ACanvas.Pen.Style := psSolid;
    faColorsh := Round(mass / mx);
    for y := 0 to mx - 1 do
    begin
      if y = mx - 1 then
        bis := mass - y * faColorsh - 1
      else
        bis := faColorsh;
      for x := 0 to bis do
      begin
        Stelle := x + y * faColorsh;
        faktor := x / bis;
        for z := 0 to 3 do
          a[z] := Trunc(b[y][z] + ((b[y + 1][z] - b[y][z]) * Faktor));
        ACanvas.Pen.Color := RGB(a[0], a[1], a[2]);
        if Horicontal then
        begin
          ACanvas.MoveTo(Rect.Left + Stelle, Rect.Top);
          ACanvas.LineTo(Rect.Left + Stelle, Rect.Bottom);
        end
        else
        begin
          ACanvas.MoveTo(Rect.Left, Rect.Top + Stelle);
          ACanvas.LineTo(Rect.Right, Rect.Top + Stelle);
        end;
      end;
    end;
    b := nil;
    ACanvas.Pen.Width := merkw;
    ACanvas.Pen.Style := merks;
    ACanvas.Pen.Color := merkp;
  end
  else
    // Please specify at least two colors
    raise EMathError.Create('Es mssen mindestens zwei Farben angegeben werden.');
end;

end.
