unit uEncode2HTMLMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Entity, StdCtrls, RxStrUtils;

type
  TfrmEncode2XMLMain = class(TForm)
    OpenDialog: TOpenDialog;
    edtInputFilename: TEdit;
    btnChooseInputFilename: TButton;
    edtOutputFilename: TEdit;
    btnChooseOutputFilename: TButton;
    lblInputFilename: TLabel;
    lblOutputFilename: TLabel;
    btnEncode: TButton;
    procedure btnChooseInputFilenameClick(Sender: TObject);
    procedure btnChooseOutputFilenameClick(Sender: TObject);
    procedure btnEncodeClick(Sender: TObject);
  private
    { Private declarations }
    procedure EncodeFile;
  public
    { Public declarations }
  end;

var
  frmEncode2XMLMain: TfrmEncode2XMLMain;

implementation

uses ConvUtils;

{$R *.dfm}

procedure TfrmEncode2XMLMain.btnChooseInputFilenameClick(Sender: TObject);
begin
  OpenDialog.Title := 'Choose file to encode';
  OpenDialog.InitialDir := '';
  if OpenDialog.Execute then
  begin
    edtInputFilename.Text := OpenDialog.FileName;
  end;
end;

procedure TfrmEncode2XMLMain.btnChooseOutputFilenameClick(Sender: TObject);
begin
  OpenDialog.Title := 'Choose file to save to';
  if (edtInputFilename.Text <> '') then
  begin
    OpenDialog.InitialDir := ExtractFilePath(edtInputFileName.Text);
  end;
  if OpenDialog.Execute then
  begin
    edtOutputFilename.Text := OpenDialog.FileName;
  end;
end;

procedure TfrmEncode2XMLMain.btnEncodeClick(Sender: TObject);
begin
  if (not FileExists(ExtractFilePath(Application.ExeName) + 'Latin1toHTML.ini')) then
  begin
    MessageDlg('No translation file found.', mtError, [mbOK], 0);
    exit;
  end;
  if (edtInputFilename.Text = '') then
  begin
    MessageDlg('Please choose a file to read from.', mtError, [mbOK], 0);
    exit;
  end;
  if (edtOutputFilename.Text = '') then
  begin
    MessageDlg('Please choose a file to write to.', mtError, [mbOK], 0);
    exit;
  end;
  btnEncode.Enabled := false;
  try
    EncodeFile;
  finally
    btnEncode.Enabled := true;
  end;
end;

procedure TfrmEncode2XMLMain.EncodeFile;
var
  InputFile: TStringList;
  HTMLCodes: TStringList;
  ActualPosition: integer;
  ConvertString: string;
  BraceOpen: boolean;      // Beginning " found.
  PosFirstTranslation: integer;
  OutputString: string;


  function EncodeToHTML(const Input: string): string;
  var
    i: integer;
  begin
    result := Input;
    for i := 0 to HTMLCodes.Count-1 do
    begin
      result := ReplaceStr(result,
                           HTMLCodes.Names[i],
                           HTMLCodes.ValueFromIndex[i]);
    end;
  end;

begin
  InputFile := TStringList.Create;
  HTMLCodes := TStringList.Create;
  try
    InputFile.LoadFromFile(edtInputFilename.Text);
    HTMLCodes.LoadFromFile(ExtractFilePath(Application.ExeName) + 'Latin1toHTML.ini');

    ActualPosition := 1;
    ConvertString := '';
    BraceOpen := false;
    OutputString := '';
    PosFirstTranslation := Pos('translation', InputFile.Text);
    while ActualPosition < Length(InputFile.Text) do
    begin
      {
        Just convert the translation items.
      }
      if (ActualPosition > PosFirstTranslation) then
      begin
        if (InputFile.Text[ActualPosition] = '"') then
        begin
          if BraceOpen then
          begin
            BraceOpen := false;
            OutputString := OutputString + '"' + EncodeToHTML(ConvertString) + '"';
            ConvertString := '';
          end
          else
          begin
            BraceOpen := true;
          end;
        end
        else
        begin
          if (not BraceOpen) then
          begin
            OutputString := OutputString + InputFile.Text[ActualPosition];
          end
          else
          begin
            ConvertString := ConvertString + InputFile.Text[ActualPosition];
          end;
        end;
      end
      else
        OutputString := OutputString + InputFile.Text[ActualPosition];

      Inc(ActualPosition);
    end;
    InputFile.Text := OutputString;
    InputFile.SaveToFile(edtOutputFilename.Text);
  finally
    FreeAndNil(InputFile);
    FreeAndNil(HTMLCodes);
  end;
  Close;
end;

end.
