(*
 *   FeedReader
 *   Copyright (C) 2003 The FeedReader Developement Group
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *)
unit XFunctions;

interface

uses
  Windows, Messages,ComCtrls,Classes,Graphics;

function EncryptDecryptWithKey(Value, Key: string): string;

procedure Delay(seconds: Longint);
procedure resize_listview;

function getflag(feedid:real;title,body:string):integer;
function html2normal(html: string): string;
function norm2xml(html: string): string;
function isbadphrase(Feedid:real;title,body:string):boolean;
Function EscapeURL( Const S: String): String;
Function UnEscapeURL(Const S: String): String;
Function EscapeHTML(Const S: String): String;
Function UnEscapeHTML(Const S: String): String;
function ishighlight(feedid:real;title,body,activefilter:string;activefeedid:real):string;
procedure add_headline_row_to_listview(i:integer;firstrow:boolean);
procedure get_fav_icon(url:string);
function GetHourfromGMT: integer;
Function XX_DecodeDate( Const OrgDate: String; Const IncludeTime: Boolean = False ): TDateTime;

function tosmall(html: string): string;

function GetFileVersion(const FileName: string; Const All: Boolean = True; Const BuildOnly: Boolean = False): string;
function HTMLPageHeader(fontname:string;fontsize:integer):string;
function get_copy_to(feedid:real;title,body:string):real;

{
  Simple encryption routines.
  Source: RX-Library
  21.11.2003  khoffrath
}
function XorEncode(const Key, Source: string): string;
function XorDecode(const Key, Source: string): string;

function ReadIEUserAgent: string;
function insertfavicon(url:string):integer;
function GetExeForProtocol(URL:string): string;

procedure OpenNewBrowserWindow(const ApplicationHandle: THandle;
                               const OpenNewWindow: boolean;
                               const Handle: HWND;
                               const Operation, FileName, Parameters, Directory: string;
                               const ShowCommand: integer);

function ForceForegroundWindow(hWnd: THandle): BOOL;

procedure EmptyKeyQueue;

function RFC822Value(const RFC822: string): Extended;

function ConvertRFC822ToInternalFormat(const RFC822: string): string;
function ConvertRDFDateTimeToInternalFormat(const RDFDateTime: string): string;

const
  DigitChars = ['0'..'9'];

implementation

uses
  httpprot, SysUtils, Forms, Registry, ShellApi,filtersform_frm,mainform_frm,misc,FReaderProperties,LanguageSupport;

//Get file version

function GetFileVersion(const FileName: string; Const All: Boolean = True; Const BuildOnly: Boolean = False): string;
var
  InfoSize, Wnd: DWORD;
  VerBuf: Pointer;
  FI: PVSFixedFileInfo;
  VerSize: DWORD;
begin
  Result := '';
  InfoSize := GetFileVersionInfoSize(PChar(FileName), Wnd);
  if InfoSize <> 0 then
  begin
    GetMem(VerBuf, InfoSize);
    try
      if GetFileVersionInfo(PChar(FileName), Wnd, InfoSize, VerBuf) then
        if VerQueryValue(VerBuf, '\', Pointer(FI), VerSize) then Begin
          If All Then
            Result := Format('%d.%d%d',
               [FI.dwFileVersionMS shr 16 and $0000FFFF,
                FI.dwFileVersionMS and $0000FFFF,
                FI.dwFileVersionLS shr 16 and $0000FFFF,
                FI.dwFileVersionLS and $0000FFFF])
          Else Begin
            If BuildOnly Then
              Result := Format('%d', [FI.dwFileVersionLS and $0000FFFF])
             Else
              Result := Format('%d.%d',
                 [FI.dwFileVersionMS shr 16 and $0000FFFF,
                  FI.dwFileVersionMS and $0000FFFF]);
          End;
        end;
    finally
      FreeMem(VerBuf);
    end;
  end;
end;
//Encrypt and decrypt passwords in subscriptions.xml file

function EncryptDecryptWithKey(Value, Key: string): string;
var
  IX, IY: Integer;
  rString: string;
begin
  iy := 1;
  rString := '';
  for ix := 1 to Length(Value) do
  begin
    rString := rString + chr(ord(Value[ix]) xor ord(Key[iy]));
    inc(iy);
    if IY > Length(Key) then
      IY := 1;
  end;
  result := rString;
end;




//Delay function

procedure Delay(seconds: Longint);
var
  WaitSecs: DWORD;
  FirstTickCount: DWORD;
begin
  WaitSecs := seconds * 1000;
  FirstTickCount := GetTickCount;
  repeat
    Application.ProcessMessages;
  until ((FirstTickCount > GetTickCount) or
    ((GetTickCount - FirstTickCount) >= WaitSecs));
end;


function GetHourfromGMT: integer;
var
  lpTimeZoneInformation: TTimeZoneInformation;
  TZMin: Integer;
begin
  TZMin := 0;

  case GetTimeZoneInformation(lpTimeZoneInformation) of

    TIME_ZONE_ID_UNKNOWN: TZMin := lpTimeZoneInformation.Bias;
    TIME_ZONE_ID_STANDARD: TZMin := lpTimeZoneInformation.Bias +
      lpTimeZoneInformation.StandardBias;
    TIME_ZONE_ID_DAYLIGHT: TZMin := lpTimeZoneInformation.Bias {+ lpTimeZoneInformation.standardBias};

//    TIME_ZONE_ID_DAYLIGHT    : TZMin := lpTimeZoneInformation.Bias + lpTimeZoneInformation.DaylightBias;
  end;

  Result := TZMin div 60;
end;

function tosmall(html: string): string;
begin
  html := stringreplace(html, '', '', [rfReplaceAll]);
  html := stringreplace(html, '', '', [rfReplaceAll]);

  html := stringreplace(html, '', '', [rfReplaceAll]);
  html := stringreplace(html, '', '', [rfReplaceAll]);
  tosmall := html;
end;

function html2normal(html: string): string;
begin
  html := UnescapeHTML( html );

  html := stringreplace(html, '</p><br>', '</p>', [rfReplaceAll, rfIgnoreCase]);
  html := stringreplace(html, '<br><p>', '<p>', [rfReplaceAll, rfIgnoreCase]);
  html2normal := html;
end;

function norm2xml(html: string): string;
begin
  html := stringreplace(html, '<', '&lt;', [rfReplaceAll]);
  html := stringreplace(html, '>', '&gt;', [rfReplaceAll]);
  norm2xml := html;
end;

Function EscapeURL( Const S: String): String;
Var
  i: Integer;
Begin
  Result := '';
  For i := 1 To Length(S) Do
    Result := Result + '%' + Format('%2x', [Ord(S[i])]);
End;


Function UnEscapeURL(Const S: String): String;
Var
  Temp, Start, HexStr: String;
  P: Integer;
Begin
  Temp := S;
  Start := '';

  Temp := StringReplace( Temp, '+', #32, [rfReplaceAll] );

  P := Pos('%', Temp);

  While P > 0 Do Begin
    Start := Start + Copy(Temp, 1, P-1);
    HexStr := Copy(Temp, P + 1, 2);
    Start := Start + Chr(StrToInt('$' + HexStr));

    Temp := Copy(Temp, P + 3, Length(Temp) - 3);
    P := Pos('%', Temp);
  End;

  Result := Start + Temp;
End;


Const
  ShyChar = #1; {character used to represent soft-hyphen in strings}
  NbspChar = #32; //#2; {character used to represent no-break space in strings}
  HtmlCodes: Array[0..123] Of Record
    Name: String;
    Value: Char;
  end =
    (
    (Name:'shy'; Value:ShyChar),
    (Name:'nbsp'; Value:NbspChar),
    (Name:'amp'; Value:'&'),
    (Name:'frasl'; Value:'/'),
    (Name:'gt'; Value:'>'),
    (Name:'lt'; Value:'<'),
    (Name:'quot'; Value:'"'),
    (Name:'minus'; Value:'-'),
    (Name:'circ'; Value:'^'),
    (Name:'tilde'; Value:'~'),
    (Name:'fnof'; Value:#131),
    (Name:'hellip'; Value:#133),
    (Name:'dagger'; Value:#134),
    (Name:'Dagger'; Value:#135),
    (Name:'permil'; Value:#137),
    (Name:'Scaron'; Value:#138),
    (Name:'lsaquo'; Value:#139),
    (Name:'OElig'; Value:#140),
    (Name:'lsquo'; Value:#145),
    (Name:'rsquo'; Value:#146),
    (Name:'ldquo'; Value:#147),
    (Name:'rdquo'; Value:#148),
    (Name:'bull'; Value:#149),
{
    Charcode #150 gives a strange character in the Delphi string and the
    translation doesn't work.
    (Name:'ndash'; Value:#150),
}
    (Name:'ndash'; Value:'-'),
    (Name:'mdash'; Value:#151),
    (Name:'trade'; Value:#153),
    (Name:'scaron'; Value:#154),
    (Name:'rsaquo'; Value:#155),
    (Name:'oelig'; Value:#156),
    (Name:'Yuml'; Value:#159),
    (Name:'iexcl'; Value:#161),
    (Name:'cent'; Value:#162),
    (Name:'pound'; Value:#163),
    (Name:'curren'; Value:#164),
    (Name:'yen'; Value:#165),
    (Name:'brvbar'; Value:#166),
    (Name:'sect'; Value:#167),
    (Name:'uml'; Value:#168),
    (Name:'copy'; Value:#169),
    (Name:'ordf'; Value:#170),
    (Name:'laquo'; Value:#171),
    (Name:'not'; Value:#172),
    (Name:'reg'; Value:#174),
    (Name:'macr'; Value:#175),
    (Name:'deg'; Value:#176),
    (Name:'plusmn'; Value:#177),
    (Name:'sup2'; Value:#178),
    (Name:'sup3'; Value:#179),
    (Name:'acute'; Value:#180),
    (Name:'micro'; Value:#181),
    (Name:'para'; Value:#182),
    (Name:'middot'; Value:#183),
    (Name:'cedil'; Value:#184),
    (Name:'sup1'; Value:#185),
    (Name:'ordm'; Value:#186),
    (Name:'raquo'; Value:#187),
    (Name:'frac14'; Value:#188),
    (Name:'frac12'; Value:#189),
    (Name:'frac34'; Value:#190),
    (Name:'iquest'; Value:#191),
    (Name:'Agrave'; Value:#192),
    (Name:'Aacute'; Value:#193),
    (Name:'Acirc'; Value:#194),
    (Name:'Atilde'; Value:#195),
    (Name:'Auml'; Value:#196),
    (Name:'Aring'; Value:#197),
    (Name:'AElig'; Value:#198),
    (Name:'Ccedil'; Value:#199),
    (Name:'Egrave'; Value:#200),
    (Name:'Eacute'; Value:#201),
    (Name:'Ecirc'; Value:#202),
    (Name:'Euml'; Value:#203),
    (Name:'Igrave'; Value:#204),
    (Name:'Iacute'; Value:#205),
    (Name:'Icirc'; Value:#206),
    (Name:'Iuml'; Value:#207),
    (Name:'ETH'; Value:#208),
    (Name:'Ntilde'; Value:#209),
    (Name:'Ograve'; Value:#210),
    (Name:'Oacute'; Value:#211),
    (Name:'Ocirc'; Value:#212),
    (Name:'Otilde'; Value:#213),
    (Name:'Ouml'; Value:#214),
    (Name:'times'; Value:#215),
    (Name:'Oslash'; Value:#216),
    (Name:'Ugrave'; Value:#217),
    (Name:'Uacute'; Value:#218),
    (Name:'Ucirc'; Value:#219),
    (Name:'Uuml'; Value:#220),
    (Name:'Yacute'; Value:#221),
    (Name:'THORN'; Value:#222),
    (Name:'szlig'; Value:#223),
    (Name:'agrave'; Value:#224),
    (Name:'aacute'; Value:#225),
    (Name:'acirc'; Value:#226),
    (Name:'atilde'; Value:#227),
    (Name:'auml'; Value:#228),
    (Name:'aring'; Value:#229),
    (Name:'aelig'; Value:#230),
    (Name:'ccedil'; Value:#231),
    (Name:'egrave'; Value:#232),
    (Name:'eacute'; Value:#233),
    (Name:'ecirc'; Value:#234),
    (Name:'euml'; Value:#235),
    (Name:'igrave'; Value:#236),
    (Name:'iacute'; Value:#237),
    (Name:'icirc'; Value:#238),
    (Name:'iuml'; Value:#239),
    (Name:'eth'; Value:#240),
    (Name:'ntilde'; Value:#241),
    (Name:'ograve'; Value:#242),
    (Name:'oacute'; Value:#243),
    (Name:'ocirc'; Value:#244),
    (Name:'otilde'; Value:#245),
    (Name:'ouml'; Value:#246),
    (Name:'divide'; Value:#247),
    (Name:'oslash'; Value:#248),
    (Name:'ugrave'; Value:#249),
    (Name:'uacute'; Value:#250),
    (Name:'ucirc'; Value:#251),
    (Name:'uuml'; Value:#252),
    (Name:'yacute'; Value:#253),
    (Name:'thorn'; Value:#254),
    (Name:'yuml'; Value:#255));

Function ParseHtmlConstant(Const S: String): Char;
Var
  i, err : Integer;
Begin
  Result := ' ';
  if (length(S) > 1) and (S[1] in ['$', '0'..'9']) then begin
    val(S, i, err);
    if (Err = 0) and (I >= 32) and (I <= 255) then
      Result := chr(i);
  end else
    for i := Low(HtmlCodes) to High(HtmlCodes) do
      if S = HtmlCodes[i].Name then begin
        Result := HtmlCodes[i].Value;
        break;
      end;
End;


procedure ExpandEscapes(var S: string);
{- returns the string with & escapes expanded}
var
  i, j : Integer;
  Co : String;
  Ch : Char;
begin
  i := length(S);
  while i > 0 do begin
    if S[i] = '&' then begin
      j := i;
      while (j < length(S)) and not (S[j] in [';',' ']) do
        inc(j);
      Co := copy(S, i + 1, j - i - 1);
      if Co <> '' then begin
        if Co[1] = '#' then begin
          Delete(Co, 1, 1);
          if UpCase(Co[1]) = 'X' then begin
            Delete(Co, 1, 1);
            Insert('$', Co, 1);
          end;
        end;
        Ch := ParseHtmlConstant(Co);
        Delete(S, i, j - i + 1);
        Insert(Ch, S, i);
      end;
    end;
    dec(i);
  end;
end;


Function UnEscapeHTML(Const S: String): String;
var
  P : Integer;
begin
  Result := S;
  P := Pos('&', S);
  if P <> 0 then
    ExpandEscapes(Result);
end;


Function EscapeHTML(Const S: String): String;
Var
  i: Integer;
  j: Integer;
  Found: Boolean;
  EscStr: String;
begin
  Result := '';
  For i := 1 To Length(S) Do Begin
    Found := False;
    For j := Low(HtmlCodes) To High(HtmlCodes) Do
      If HtmlCodes[j].Value = S[i] Then Begin
        Found := True;
        EscStr := '&' + HtmlCodes[j].Name + ';';
      End;
    If Found Then
      Result := Result + EscStr
     Else
      Result := Result + S[i];
  End;
end;


Function XX_DecodeDate( Const OrgDate: String; Const IncludeTime: Boolean = False ): TDateTime;
Var
  MyDate, MyTime: TDateTime;
  tzz, dat, da1, da2: String;
  Hourshift: Integer;
  dmin, dh, dd, dm, dy: Integer;
  wdd, wdm, wdy: Word;
  kuu_paevi: Integer;
Begin

  MyDate := Now;
  MyTime := Now;

  Try
        //Siin on kuupaev vaja paika saada
        tzz := copy(OrgDate, 1 + pos('+', OrgDate), 2);
        try
          hourshift := (GetHourfromGMT * -1) - strtoint(trim(tzz));
        except
          on e: Econverterror do
          begin
            hourshift := 0;
          end;
        end;

        if (pos('-', OrgDate) > 0) and (length(OrgDate) > 11) then
        begin
          dat := copy(OrgDate, 12, 5);
          da1 := copy(dat, 1, 2);
          da2 := copy(dat, 4, 2);
          dmin := strtoint(da2);
          dh := strtoint(da1) + hourshift;
          dd := strtoint(copy(OrgDate, 9, 2));
          dm := strtoint(copy(OrgDate, 6, 2));
          dy := strtoint(copy(OrgDate, 0, 4));
          if dh >= 24 then
          begin
            dh := dh - 24;
            dd := dd + 1;
          end
          else if dh < 0 then
          begin
            dh := 24 + dh;
            dd := dd - 1;
          end;

          if (dm = 1) or (dm = 3) or (dm = 5) or (dm = 7) or (dm = 8) or (dm =
            10) or (dm = 12) then
            kuu_paevi := 31
          else if (dm = 2) then
            kuu_paevi := 28
          else
            kuu_paevi := 30;

          if dd > kuu_paevi then
          begin
            dd := dd - kuu_paevi;
            dm := dm + 1;
          end
          else if dd < 1 then
          begin
            dd := kuu_paevi - dd;
            dm := dm - 1;
          end;
          if dm > 12 then
          begin
            dm := dm - 12;
            dy := dy + 1;
          end
          else if dm < 1 then
          begin
            dm := 12 - dm;
            dy := dy - 1;
          end;

          if dh < 0 then
            dh := 24 + dh;
          try
            mydate := encodedate(dy, dm, dd);
          except
          end;
          mytime := encodetime(dh, dmin, 0, 0);
        End;

  Except
    MyDate := Now;
    MyTime := Now;
  End;

  If Not IncludeTime Then Begin
    DecodeDate( MyDate, wdy, wdm, wdd );
    MyDate := EncodeDate( wdy, wdm, wdd );
  End
   Else MyDate := MyDate + MyTime;

  Result := MyDate;
End;
function HTMLPageHeader(fontname:string;fontsize:integer):string;
var tmpstring,browserstylesheet:string;
begin
tmpstring := '<html><style type="text/css">body {margin:0; padding:0; background:#fff; color:#000}';
  browserstylesheet := 'td,body,.form {font-family:' + FontName +
    '; letter-spacing:1.1lm;font-size: ' + inttostr(FontSize) + 'pt}';
  browserstylesheet := browserstylesheet +
    'A {color:#336699; text-decoration: none;}';
  tmpstring := tmpstring + browserstylesheet;
  tmpstring := tmpstring + '</style>';
  result:=tmpstring;
end;

function XorEncode(const Key, Source: string): string;
var
  I: Integer;
  C: Byte;
begin
  Result := '';
  for I := 1 to Length(Source) do begin
    if Length(Key) > 0 then
      C := Byte(Key[1 + ((I - 1) mod Length(Key))]) xor Byte(Source[I])
    else
      C := Byte(Source[I]);
    Result := Result + AnsiLowerCase(IntToHex(C, 2));
  end;
end;

function XorDecode(const Key, Source: string): string;
var
  I: Integer;
  C: Char;
begin
  Result := '';
  for I := 0 to Length(Source) div 2 - 1 do begin
    C := Chr(StrToIntDef('$' + Copy(Source, (I * 2) + 1, 2), Ord(' ')));
    if Length(Key) > 0 then
      C := Chr(Byte(Key[1 + (I mod Length(Key))]) xor Byte(C));
    Result := Result + C;
  end;
end;

{-----------------------------------------------------------------------------
  Procedure: ReadIEUserAgent
  Purpose  : Reads the user agent for IE from the registry and returns it.

  Author   : Karsten Hoffrath (khoffrath)
  Arguments: None
  Result   : string
  History  :
    27.11.2003 21:41:51
-----------------------------------------------------------------------------}
function ReadIEUserAgent: string;
var
  Reg: TRegistry;
begin

  {
    Set a generic user agent.
  }
  result := 'Mozilla/4.0 (compatible; MSIE 5.0; Win32)';
  try

    {
      Create a "readonly" registry object.
      Since we just want to read from the registry we set the access level to
      KEY_READ.
      The creation would fail if the user has no administrator rights on the pc.
    }
    Reg := TRegistry.Create(KEY_READ);
    try

      {
        Set the path to the user agent entry.
      }
      Reg.RootKey := HKEY_CURRENT_USER;
      Reg.OpenKey('\Software\Microsoft\Windows\CurrentVersion\Internet Settings\', false);
      {
        If the value exists and its not empty read and return it.
      }
      if (Reg.ValueExists('User Agent')) and not (Length(Trim(Reg.ReadString('User Agent'))) = 0)
        then result := Reg.ReadString('User Agent');
    finally
      FreeAndNil(Reg);
    end;
  except
  end;
end;


{-----------------------------------------------------------------------------
  Procedure: GetExeForProtocol
  Purpose  : Returns the exe file associated with the protocol used in URL

   Author   : Simon Reinhardt (reinhardt@picsoft.de)
  Arguments: URL:string
  Result   : string
  History  :
    09.12.2003 14:33:57  khoffrath
      Added for retrieving the default webbrowser (this method is public domain).
-----------------------------------------------------------------------------}
function GetExeForProtocol(URL:string):string;
var Reg  : TRegistry;
    Res  : boolean;
    Temp : string;
    P    : integer;

  function PosN(const AText,S:string;N:integer):integer;
  var Temp : string;
      P    : integer;
  begin
    if N>1 then begin
      Temp:=copy(S, N, length(S)-N+1);
      dec(N);
    end
    else
      Temp:=S;
    P:=Pos(AText, Temp);
    if P>0 then
      Result:=P+N
    else
      Result:=0;
  end;

  function GetWindowsDir:string;
  var WinDir : array [0..Max_Path] of char;
  begin
    GetWindowsDirectory(WinDir, Max_Path);
    Result:=IncludeTrailingPathDelimiter(String(WinDir));
  end;

  function GetProgramFilesDir:string;
  var Reg : TRegistry;
      Res : boolean;
  begin
    Reg:=TRegistry.Create;
    try
      Reg.Rootkey:=HKEY_LOCAL_MACHINE;
      Res:=Reg.OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion',false);
      if Res then begin
        Result:=IncludeTrailingPathDelimiter(Reg.ReadString('ProgramFilesDir'));
        Reg.CloseKey;
      end
      else
        Result:='';
    finally
      Reg.Free;
    end;
  end; 

  function ReplaceRegistryPathVars(const AText:string):string;
  var P,Q      : integer;
      PathVar,
      RealPath : string;

    function GetProgramFilesPathVar:string;
    var Reg : TRegistry;
        Res : boolean;
    begin
      Result:='%ProgramFiles%';
      Reg:=TRegistry.Create;
      try
        Reg.Rootkey:=HKEY_LOCAL_MACHINE;
        Res:=Reg.OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion',false);
        if Res then begin
          Result:=Reg.ReadString('ProgramFilesPath');
          Reg.CloseKey;
        end;
      finally
        Reg.Free;
      end;
    end;

  begin
    Result:=AText;
    P:=Pos('%', AText);
    if P>0 then begin
      Q:=PosN('%', AText, P+1);
      PathVar:=LowerCase(copy(AText, P, Q-P+1));
      RealPath:='???';
      if PathVar=LowerCase(GetProgramFilesPathVar) then
        RealPath:=GetProgramFilesDir;
      if PathVar='%systemroot%' then
        RealPath:=GetWindowsDir;
      Result:=copy(AText, 1, P-1)+ExcludeTrailingPathDelimiter(RealPath)+copy(AText, Q+1, length(AText)-Q);
    end;
  end;

begin
  Result:='';
  P:=Pos(':', URL);
  if P>1 then
    delete(URL, P, length(URL)-P+1);
  {$IFDEF SR_Delphi4_Up}
  Reg := TRegistry.Create(Key_Read);
  {$ELSE}
  Reg := TRegistry.Create;
  {$ENDIF}
  try
    Reg.Rootkey:=HKEY_CLASSES_ROOT;
    Res:=Reg.OpenKey(URL+'\shell\open\command', false);
    if Res then begin
      Temp:=Reg.ReadString('');
      while (length(Temp)>0) and ((Temp[1]='"') or (Temp[1]=' ')) do
        delete(Temp, 1, 1);
      P:=Pos('"', Temp);
      if P>0 then
        delete(Temp, P, length(Temp)-P+1);
      Result:=ReplaceRegistryPathVars(Temp);
      Reg.CloseKey;
    end;
  finally
    Reg.Free;
  end;
end; {GetExeForProtocol}

{-----------------------------------------------------------------------------
  Procedure: OpenNewBrowserWindow
  Purpose  : Shows the FileName in a webbrowser.
    If OpenNewWindow is true, a new window is opened, otherwise windows will
    reuse existing browsers.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: const OpenNewWindow: boolean; const Handle: HWND;
             const Operation, FileName, Parameters, Directory: string;
             const ShowCommand: integer
  Result   : None
  History  :
    09.12.2003 17:37:23
-----------------------------------------------------------------------------}
procedure OpenNewBrowserWindow(const ApplicationHandle: THandle;
                               const OpenNewWindow: boolean;
                               const Handle: HWND;
                               const Operation, FileName, Parameters, Directory: string;
                               const ShowCommand: integer);
var
  AppPath: String;
  AppExe: String;
  ExePath: string;
  pOperation: PChar;
  pFilename: PChar;
  pParameter: PChar;
  pDirectory: PChar;
begin
  if (Operation <> '') then
    pOperation := PChar(Operation)
  else
    pOperation := nil;
  if (FileName <> '') then
    pFilename := PChar(FileName)
  else
    pFilename := nil;
  if (Parameters <> '') then
    pParameter := PChar(Parameters)
  else
    pParameter := nil;
  if (Directory <> '') then
    pDirectory := PChar(Directory)
  else
    pDirectory := nil;

  if OpenNewWindow then
  begin
    {
      Open a new browser window.
    }
    ExePath := GetExeForProtocol(FileName);
    AppPath := ExtractFilePath(ExePath);
    AppExe := ExtractFileName(ExePath);
    {
      Mozilla has ' -url ' appended in ExePath.
      With this string shellexecute doesn't open a new window.
      Therefor delete it.
    }
    if Pos(' ', AppExe) <> 0 then
      Delete(AppExe, Pos(' ', AppExe), Length(AppExe));
    ExePath := AppPath + AppExe;
    ShellExecute(Handle, pOperation, PChar(ExePath), pFilename,
                 pDirectory, ShowCommand);
  end
  else
  begin
    ShellExecute(Handle, pOperation, pFilename, pParameter, pDirectory, ShowCommand);
  end;
end;

{-----------------------------------------------------------------------------
  Procedure: ForceForegroundWindow
  Purpose  : Force the given window in the foreground.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: hWnd: THandle
  Result   : BOOL
  History  :
    13.12.2003 16:37:33
-----------------------------------------------------------------------------}
function ForceForegroundWindow(hWnd: THandle): BOOL;
const
  SPI_GETFOREGROUNDLOCKTIMEOUT = $2000;
  SPI_SETFOREGROUNDLOCKTIMEOUT = $2001;
var
  timeout: DWORD;
begin
  if ((Win32Platform = VER_PLATFORM_WIN32_NT) and (Win32MajorVersion > 4))
    or ((Win32Platform = VER_PLATFORM_WIN32_WINDOWS) and
       ((Win32MajorVersion > 4) or ((Win32MajorVersion = 4) and
        (Win32MinorVersion > 0)))) then
  begin
    SystemParametersInfo(SPI_GETFOREGROUNDLOCKTIMEOUT, 0, @timeout, 0);
    SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT, 0, TObject(0), SPIF_SENDCHANGE);
    Result := SetForegroundWindow(hWnd);
    SystemParametersInfo(SPI_SETFOREGROUNDLOCKTIMEOUT, 0, TObject(timeout), SPIF_SENDCHANGE);
  end
  else Result := SetForegroundWindow(hWnd);
end;

{-----------------------------------------------------------------------------
  Procedure: EmptyKeyQueue
  Purpose  : Remove all key messages from the applications message queue.
    Used to remove VK_ESCAPE messages. If another window is closed with Esc,
    then remove this keypress-message so that the main window is not minimized.
  Author   : Karsten Hoffrath (khoffrath)
  Arguments: None
  Result   : None
  History  :
    21.12.2003 15:05:00
-----------------------------------------------------------------------------}
procedure EmptyKeyQueue;
var
  Msg: TMsg;
begin
  while PeekMessage(Msg, Application.MainForm.Handle, WM_KEYFIRST, WM_KEYLAST, PM_REMOVE or PM_NOYIELD )
    do ;
end;

{-----------------------------------------------------------------------------
  Procedure: RFC822Value
  Purpose  :
    Returns the timeshift for the given timezone.
    See http://asg.web.cmu.edu/rfc/rfc822.html for timezone definitions.
  Author   : Karsten Hoffrath (khoffrath)
  History  :
    08.04.2004  khoffrath
-----------------------------------------------------------------------------}
function RFC822Value(const RFC822: string): Extended;
var
	S: string;
	L: LongInt;
  Index: integer;
  HourValue: string;
begin
	S := UpperCase(Trim(RFC822));
	Result := 0.0;

  Index := Pos('+', S);
  if (Index = 0) then Index := Pos('-', S);
  if (Index <> 0) then
  begin
    HourValue := Copy(S, Index, Length(S));
    Delete(S, Index, Length(S));
  end;
	if (S = 'GMT') or (S = 'UT') or (S = 'Z') or (S = '') then
  begin
    if (HourValue <> '') then
    begin
      try
        Result := Result + StrToInt(HourValue);
      except
      end;
    end;
    Exit;
  end;
  
  if (S = 'M') then	Result := -12.0
	else if (S = 'PST') then Result := -8.0
	else if (S = 'MST') or (S = 'PDT') then	Result := -7.0
	else if (S = 'CST') or (S = 'MDT') then	Result := -6.0
	else if (S = 'EST') or (S = 'CDT') then	Result := -5.0
	else if (S = 'EDT') then Result := -4.0
	else if (S = 'A') then Result := -1.0
	else if (S = 'N') then Result := 1.0
	else if (S = 'Y') then Result := 12.0
	else
	begin
    try
      L := StrToInt (S);
    except
      L := 0;
    end;
		Result := L div 100 + (L mod 100) / 60;
	end;

  if (HourValue <> '') then
  begin
    try
      Result := Result + StrToInt(HourValue);
    except
    end;
  end;
end;


{-----------------------------------------------------------------------------
  Procedure: ConvertRFC822ToInternalFormat
  Purpose  :
    Tries to convert the RFC822 date to the internally used format.
  Author   : Karsten Hoffrath (khoffrath)
  History  :
    08.04.2004  khoffrath
-----------------------------------------------------------------------------}
function ConvertRFC822ToInternalFormat(const RFC822: string): string;
var
  s: string;
  Tmp: string;
  Day, Month, Year: word;
  Hour, Min, Sec, MSec: word;
  TimeZone: string;
begin
  if (Length(RFC822) > 0) then
  begin
    if (length(RFC822) <> 25) then
    begin
      s := RFC822;
      try
        {
          Remove the day of week.
        }
        if (Pos(' ', s) <> 0) then Delete(s, 1, Pos(' ', s));

        {
          Extract the day.
        }
        Day := StrToInt(Copy(s, 1, Pos(' ', s)-1));
        Delete(s, 1, Pos(' ', s));

        {
          Extract the month and convert it to an integer.
        }
        Tmp := Copy(s, 1, Pos(' ', s)-1);
        Delete(s, 1, Pos(' ', s));

        if (Tmp = 'Jan') then Month := 1
          else if (Tmp = 'Feb') then Month := 2
          else if (Tmp = 'Mar') then Month := 3
          else if (Tmp = 'Apr') then Month := 4
          else if (Tmp = 'May') then Month := 5
          else if (Tmp = 'Jun') then Month := 6
          else if (Tmp = 'Jul') then Month := 7
          else if (Tmp = 'Aug') then Month := 8
          else if (Tmp = 'Sep') then Month := 9
          else if (Tmp = 'Oct') then Month := 10
          else if (Tmp = 'Nov') then Month := 11
          else if (Tmp = 'Dec') then Month := 12;

        {
          Extract the year.
        }
        Year := StrToInt(Copy(s, 1, Pos(' ', s)-1));
        if Year<1000 then Year:=Year+2000;
        Delete(s, 1, Pos(' ', s));

        {
          Extract the time.
        }
        Hour := StrToInt(Copy(s, 1, Pos(':', s)-1));
        Delete(s, 1, Pos(':', s));
        Min := StrToInt(Copy(s, 1, Pos(':', s)-1));
        Delete(s, 1, Pos(':', s));
        Sec := StrToInt(Copy(s, 1, Pos(' ', s)-1));
        Delete(s, 1, Pos(' ', s));

        {
          Extract the timezone.
        }
        if (Pos(':', s) <> 0) then
          Timezone := Copy(s, 1, Pos(':', s)-1)
        else
          Timezone := s;


        result := IntToStr(Year) + '-' + Format('%.2d', [Month]) + '-' + Format('%.2d', [Day])
                  + 'T'
                  + Format('%.2d', [Hour]) + ':' + Format('%.2d', [Min]) + ':' + Format('%.2d', [Sec])
                  + '+'
                  + IntToStr(Trunc(RFC822Value(Timezone)));

      except
        {
          In case of an exception return the actual date/time.
        }
        DecodeDate(SysUtils.Date, Year, Month, Day);
        DecodeTime(sysutils.Time, Hour, Min, Sec, MSec);
        result := IntToStr(Year) + '-' + Format('%.2d', [Month]) + '-' + Format('%.2d', [Day]) + 'T' + Format('%.2d', [Hour]) + ':' + Format('%.2d', [Min]) + ':' + Format('%.2d', [Sec]);
        if (GetHourfromGMT * -1) >= 0 then
          result := result + '+' + Format('%.2d', [GetHourfromGMT * -1]) + ':00'
        else
          result := result + '-' + Format('%.2d', [GetHourfromGMT * -1]) + ':00';
      end;
    end
    else
      result := RFC822;
  end
  else
  begin
    DecodeDate(SysUtils.Date, Year, Month, Day);
    DecodeTime(sysutils.Time, Hour, Min, Sec, MSec);
    result := IntToStr(Year) + '-' + Format('%.2d', [Month]) + '-' + Format('%.2d', [Day]) + 'T' + Format('%.2d', [Hour]) + ':' + Format('%.2d', [Min]) + ':' + Format('%.2d', [Sec]);
    if (GetHourfromGMT * -1) >= 0 then
      result := result + '+' + Format('%.2d', [GetHourfromGMT * -1]) + ':00'
    else
      result := result + '-' + Format('%.2d', [GetHourfromGMT * -1]) + ':00';
  end;
end;

{-----------------------------------------------------------------------------
  Procedure: ConvertRDFDateTimeToInternalFormat
  Purpose  :
    Tries to convert the RDF date to the internally used format.
  Author   : Karsten Hoffrath (khoffrath)
  History  :
    08.04.2004  khoffrath
-----------------------------------------------------------------------------}
function ConvertRDFDateTimeToInternalFormat(const RDFDateTime: string): string;
var
  s: string;
  Day, Month, Year: word;
  Hour, Min, Sec, MSec: word;
  TimeZone: string;
begin
  if (Length(RDFDateTime) > 0) then
  begin
    s := RDFDateTime;
    try

      {
        Extract the year.
      }
      Year := StrToInt(Copy(s, 1, Pos('-', s)-1));
      Delete(s, 1, Pos('-', s));

      {
        Extract the month.
      }
      Month := StrToInt(Copy(s, 1, Pos('-', s)-1));
      Delete(s, 1, Pos('-', s));

      {
        Extract the day.
      }
      Day := StrToInt(Copy(s, 1, Pos('T', s)-1));
      Delete(s, 1, Pos('T', s));

      {
        Extract the time.
      }
      Hour := StrToInt(Copy(s, 1, Pos(':', s)-1));
      Delete(s, 1, Pos(':', s));
      Min := StrToInt(Copy(s, 1, 2));
      Delete(s, 1, 2);

      {
        Extract the timezone.
      }
      if (Pos(':', s) <> 0) then
        Timezone := Copy(s, 1, Pos(':', s)-1)
      else
        Timezone := s;


      result := IntToStr(Year) + '-' + Format('%.2d', [Month]) + '-' + Format('%.2d', [Day])
                + 'T'
                + Format('%.2d', [Hour]) + ':' + Format('%.2d', [Min]) + ':' + Format('%.2d', [Sec])
                + '+'
                + Timezone;
    except
      {
        In case of an exception return the actual date/time.
      }
      DecodeDate(SysUtils.Date, Year, Month, Day);
      DecodeTime(sysutils.Time, Hour, Min, Sec, MSec);
      result := IntToStr(Year) + '-' + Format('%.2d', [Month]) + '-' + Format('%.2d', [Day]) + 'T' + Format('%.2d', [Hour]) + ':' + Format('%.2d', [Min]) + ':' + Format('%.2d', [Sec]);
      if (GetHourfromGMT * -1) >= 0 then
        result := result + '+' + Format('%.2d', [GetHourfromGMT * -1]) + ':00'
      else
        result := result + '-' + Format('%.2d', [GetHourfromGMT * -1]) + ':00';
    end;
  end
  else
  begin
    DecodeDate(SysUtils.Date, Year, Month, Day);
    DecodeTime(sysutils.Time, Hour, Min, Sec, MSec);
    result := IntToStr(Year) + '-' + Format('%.2d', [Month]) + '-' + Format('%.2d', [Day]) + 'T' + Format('%.2d', [Hour]) + ':' + Format('%.2d', [Min]) + ':' + Format('%.2d', [Sec]);
    if (GetHourfromGMT * -1) >= 0 then
      result := result + '+' + Format('%.2d', [GetHourfromGMT * -1]) + ':00'
    else
      result := result + '-' + Format('%.2d', [GetHourfromGMT * -1]) + ':00';
  end;
end;

function get_copy_to(feedid:real;title,body:string):real;
var i,j:integer;
begin
//
get_copy_to:=-1;
for i:=0 to filtersform.listview1.items.Count-1 do with filtersform do
    begin
  if ((feednumber(filtersform.listview1.items[i].data).id=0) or (feedid=feednumber(filtersform.listview1.items[i].data).id))
    and (feednumber(listview1.items[i].data).action=3)
    and ((pos(lowercase(listview1.Items[i].subitems[1]),lowercase(title))>0) or (pos(lowercase(listview1.Items[i].subitems[1]),lowercase(body))>0)) then
    begin
    For j := Low(gSubscribedFeeds) To High(gSubscribedFeeds) Do
        if (gSubscribedFeeds[j].feedtype='localfeed') and (gSubscribedFeeds[j].title=listview1.Items[i].SubItems[3]) then
        begin
        get_copy_to:=gSubscribedfeeds[j].feedid;
        break;
        end;
    end;
    end;



end;
function getflag(feedid:real;title,body:string):integer;
//this function returns flag number based on filters
var i:integer;
  begin
  getflag:=0;
for i:=0 to filtersform.listview1.items.Count-1 do with filtersform do
    begin
  if ((feednumber(filtersform.listview1.items[i].data).id=0) or (feedid=feednumber(filtersform.listview1.items[i].data).id))
    and (feednumber(listview1.items[i].data).action=2)
    and ((pos(lowercase(listview1.Items[i].subitems[1]),lowercase(title))>0) or (pos(lowercase(listview1.Items[i].subitems[1]),lowercase(body))>0)) then
    begin
    getflag:=feednumber(listview1.items[i].data).option+44;
//    getflag:=combo_flagcolors.Items.IndexOf(listview1.Items[i].SubItems[3])+44;
    end;
    end;
  end;
function isbadphrase(Feedid:real;title,body:string):boolean;
//this function returns true if some of bad words is contained in title or body
//bad word=filter where action is "delete"
var i:integer;
begin
isbadphrase:=false;
for i:=0 to filtersform.listview1.items.Count-1 do with filtersform do
  begin
  if (feednumber(listview1.items[i].data).action=1) and ((pos(lowercase(listview1.Items[i].subitems[1]),lowercase(title))>0) or (pos(lowercase(listview1.Items[i].subitems[1]),lowercase(body))>0)) then
    isbadphrase:=true;
  end;
end;
function ishighlight(feedid:real;title,body,activefilter:string;activefeedid:real):string;
var i:integer;
begin
ishighlight:='';
for i:=0 to filtersform.listview1.items.Count-1 do with filtersform do
  begin
  if ((activefeedid=feednumber(listview1.items[i].data).id))
   and ((activefilter='') or (activefilter=listview1.items[i].subitems[1]))
   and (feednumber(listview1.items[i].data).action=0)
   and ((pos(lowercase(listview1.Items[i].subitems[1]),lowercase(title))>0) or (pos(lowercase(listview1.Items[i].subitems[1]),lowercase(body))>0)) then
    begin
    ishighlight:=colorbox1.ColorNames[feednumber(listview1.items[i].data).option];
    exit;
    end;
  end;
end;
procedure get_fav_icon(url:string);
  var icostream:tfilestream;
  var icolocation,filename:string;
  var http:thttpcli;
begin
try
      http:=thttpcli.create(mainwindow);
      InitHttpClientProxySettings( http );
      icolocation:=url;
      icolocation:=stringreplace(icolocation,'http://','',[rfreplaceall]);
      icolocation:=stringreplace(icolocation,'https://','',[rfreplaceall]);
      icolocation:=copy(icolocation+'/',1,pos('/',icolocation+'/')-1);
      filename:=icolocation;
      icolocation:='http://'+icolocation+'/favicon.ico';
      http.URL:=icolocation;
      http.head;
      if http.statuscode=200 then
        begin
        try
        icostream:=tfilestream.Create(gProperties.DataDirectory+'\Favicons\'+filename+'.ico',fmCreate		);
        http.RcvdStream:=icostream;
        http.Abort;
        http.Get;
        icostream.Free;
        except
        end;
      end;
      http.Free;
except
end;
end;

procedure add_headline_row_to_listview(i:integer;firstrow:boolean);
var li:tlistitem;
  tzz, da1, da2, dat: string;
  MyDate, mytime: TDateTime;
  has_topic_row,isfolderselected:boolean;
var  kuu_paevi, tmpwidth, iii,dy, dm, dd, dh, dmin, hourshift: integer;
begin
          li := mainwindow.listview1.items.insert(0);


            if theadline(gHeadlines[i]).flag>0 then
              begin
              try
              li.ImageIndex := theadline(gHeadlines[i]).flag;
              except
              end;
              end
            else
              begin
              if theadline(gHeadlines[i]).Read <> 0 then li.ImageIndex := 6 else li.imageindex := 5;
              end;





          li.Data := theadline.create;
          li.Data := theadline(gheadlines[i]);
          //li.data:=an.data;

          //Siin on kuupaev vaja paika saada
          tzz := copy(theadline(gHeadlines[i]).date, 1 + pos('+', theadline(gHeadlines[i]).date), 2);
          try
            hourshift := (GetHourfromGMT * -1) - strtoint(trim(tzz));
          except
            on e: Econverterror do
            begin
              hourshift := 0;
            end;
          end;
          if (pos('-', theadline(gHeadlines[i]).date) > 0) and (length(theadline(gHeadlines[i]).date) > 11) then
          begin
            dat := copy(theadline(gHeadlines[i]).date, 12, 5);
            da1 := copy(dat, 1, 2);
            da2 := copy(dat, 4, 2);
            dmin := strtoint(da2);
            dh := strtoint(da1) + hourshift;
            dd := strtoint(copy(theadline(gHeadlines[i]).date, 9, 2));
            dm := strtoint(copy(theadline(gHeadlines[i]).date, 6, 2));
            dy := strtoint(copy(theadline(gHeadlines[i]).date, 0, 4));
            if dh >= 24 then
            begin
              dh := dh - 24;
              dd := dd + 1;
            end
            else if dh < 0 then
            begin
              dh := 24 + dh;
              dd := dd - 1;
            end;

            if (dm = 1) or (dm = 3) or (dm = 5) or (dm = 7) or (dm = 8) or (dm =
              10) or (dm = 12) then
              kuu_paevi := 31
            else if (dm = 2) then
              kuu_paevi := 28
            else
              kuu_paevi := 30;

            if dd > kuu_paevi then
            begin
              dd := dd - kuu_paevi;
              dm := dm + 1;
            end
            else if dd < 1 then
            begin
              dd := kuu_paevi - dd;
              dm := dm - 1;
            end;
            if dm > 12 then
            begin
              dm := dm - 12;
              dy := dy + 1;
            end
            else if dm < 1 then
            begin
              dm := 12 - dm;
              dy := dy - 1;
            end;

            if dh < 0 then
              dh := 24 + dh;
            mydate := 0;
            try
              mydate := encodedate(dy, dm, dd);
            except
            end;
            mytime := encodetime(dh, dmin, 0, 0);

            if gProperties.ShowDate And gProperties.ShowTime Then
              dat := formatdatetime('hh:nn dd.mm.', mydate + mytime)
            else if not (gProperties.ShowDate or gProperties.ShowTime) then
              dat := formatdatetime('hh:nn dd.mm.', mydate + mytime)
            else if gProperties.ShowDate and not gProperties.ShowTime then
              dat := formatdatetime('dd.mm.', mydate + mytime)
            else if gProperties.ShowTime and not gProperties.ShowDate then
              dat := formatdatetime('hh:nn', mydate + mytime);
          end;

          li.SubItems.Add(dat);
          if theadline(gHeadlines[i]).rating<=0 then
          li.SubItems.Add( UnEscapeHTML(theadline(gHeadlines[i]).tiitel))
          else
          li.SubItems.Add( UnEscapeHTML(theadline(gHeadlines[i]).tiitel)+' ('+inttostr(theadline(gHeadlines[i]).rating)+')' );
          tmpwidth := 0;


          isfolderselected:=(mainwindow.TreeView1.selected<>nil) and nodeischildof(mainwindow.TreeView1.Selected,gmyfeedstreenode) and (mainwindow.TreeView1.selected.data<>nil) and (tfeedinfo(mainwindow.TreeView1.Selected.Data).feedtype='folder');

          has_topic_row:=((theadline(gHeadlines[i]).topic <> '') and (gProperties.ShowTopics)) or isfolderselected ;

          if not has_topic_row and (mainwindow.listview1.columns.Count = 4) then
          begin
            gProperties.LVColumnWidth3 := mainwindow.listview1.Columns[3].width;
            mainwindow.listview1.Columns.Delete(3);
          end;

          if has_topic_row and (mainwindow.listview1.columns.Count < 4) then
            begin
            mainwindow.listview1.Columns.add;
            tmpwidth := gProperties.LVColumnWidth3;
            if tmpwidth=0 then tmpwidth:=10;
            mainwindow.listview1.Columns[3].width := tmpwidth;
          end;
          if mainwindow.listview1.columns.Count = 4 then
            begin
              if isfolderselected then
                mainwindow.listview1.columns[3].caption := TranslateString('Feed')
              else
                mainwindow.listview1.columns[3].caption := TranslateString('Keyword');
            end;
          if isfolderselected then
            begin
              EnterCriticalSection(gCritSect);
              try
              For iii := Low(gSubscribedFeeds) To High(gSubscribedFeeds) Do
                If gSubscribedFeeds[iii].feedid = theadline(gHeadlines[i]).owner Then Begin
                li.subitems.add(gSubscribedFeeds[iii].title);
                Break;
                End;
              finally
              LeaveCriticalSection(gCritSect);
              end;


            end
          else if not isfolderselected and (theadline(gHeadlines[i]).topic <> '') then
            li.subitems.add(theadline(gHeadlines[i]).topic);

end;
function insertfavicon(url:string):integer;
var httplocation:string;
icon:ticon;
begin
              icon:=ticon.create;
              insertfavicon:=0;
              try
              httplocation:=url;
              httplocation:=stringreplace(httplocation,'http://','',[rfreplaceall]);
              httplocation:=stringreplace(httplocation,'https://','',[rfreplaceall]);
              httplocation:=copy(httplocation+'/',1,pos('/',httplocation+'/')-1);
              if fileexists(gProperties.DataDirectory+'\Favicons\'+httplocation+'.ico') then
                begin
                try
                icon.LoadFromFile(gProperties.DataDirectory+'\Favicons\'+httplocation+'.ico');
                insertfavicon:=mainwindow.imagelist_treeview.AddIcon(icon);
                except
                //wrong format
                deletefile(gProperties.DataDirectory+'\Favicons\'+httplocation+'.ico');
                end;
                end;
              except
              end;
end;
procedure resize_listview;
var tmpwidth:integer;
begin
try
  with mainwindow.listview1 do begin

if columns.Count = 4 then
  begin
    if gproperties<>nil then
      tmpwidth := gProperties.LVColumnWidth3;
    Columns[2].Width := ClientWidth - tmpWidth - Columns[0].width - Columns[1].width - 18;

  end
else if columns.Count < 4
  then Columns[2].Width := ClientWidth -Columns[0].width - Columns[1].width - 18;

end;
except
end;

end;
end.
